@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -9,8 +9,6 @@
 #include "PWSFilters.h"
 #include "PWHistory.h"
 #include "PWSprefs.h"
-#include "Match.h"
-#include "UUIDGen.h"
 #include "core.h"
 #include "PWScore.h"
 #include "StringX.h"
@@ -34,10 +32,12 @@
 #include <iomanip>
 #include <string>
 #include <vector>
+#include <functional>
 #include <algorithm>
 #include <map>
 
 using namespace std;
+using pws_os::CUUID;
 
 typedef std::vector<stringT>::const_iterator vciter;
 typedef std::vector<stringT>::iterator viter;
@@ -102,11 +102,11 @@ static void GetFilterTestXML(const st_FilterRow &st_fldata,
     case PWSMatch::MT_DATE:
     {
       if (st_fldata.fdatetype == 0 /* DTYPE_ABS */) {
-        const StringX tmp1 = PWSUtil::ConvertToDateTimeString(st_fldata.fdate1, TMC_XML);
+        const StringX tmp1 = PWSUtil::ConvertToDateTimeString(st_fldata.fdate1, PWSUtil::TMC_XML);
         utf8conv.ToUTF8(tmp1.substr(0, 10), utf8, utf8Len);
         oss << sztab5 << "<date1>" << utf8
                                                 << "</date1>" << szendl;
-        const StringX tmp2 = PWSUtil::ConvertToDateTimeString(st_fldata.fdate2, TMC_XML);
+        const StringX tmp2 = PWSUtil::ConvertToDateTimeString(st_fldata.fdate2, PWSUtil::TMC_XML);
         utf8conv.ToUTF8(tmp2.substr(0, 10), utf8, utf8Len);
         oss << sztab5 << "<date2>" << utf8
                                                 << "</date2>" << szendl;
@@ -139,6 +139,10 @@ static void GetFilterTestXML(const st_FilterRow &st_fldata,
       oss << sztab5 << "<dca>" << st_fldata.fdca 
                                               << "</dca>" << szendl;
       break;
+    case PWSMatch::MT_SHIFTDCA:
+      oss << sztab5 << "<shiftdca>" << st_fldata.fdca 
+                                              << "</shiftdca>" << szendl;
+      break;
     case PWSMatch::MT_ENTRYSTATUS:
     {
       // Get index for string values
@@ -191,7 +195,13 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
     szendl = "\0";
   }
 
-  utf8conv.ToUTF8(filters.fname.c_str(), utf8, utf8Len);
+  // Filter name is an element attribute and so must be a quoted string.
+  // Convert any embedded quotes to &quot;
+  stringT fname = filters.fname;
+  const stringT from = _T("\""), to = _T("&quot;");
+  Replace(fname, from, to);
+
+  utf8conv.ToUTF8(fname.c_str(), utf8, utf8Len);
   oss << sztab1 << "<filter filtername=\"" << reinterpret_cast<const char *>(utf8) 
       << "\">" << szendl;
 
@@ -243,6 +253,9 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
       case FT_DCA:
         pszfieldtype = "DCA";
         break;
+      case FT_SHIFTDCA:
+        pszfieldtype = "ShiftDCA";
+        break;
       case FT_EMAIL:
         pszfieldtype = "email";
         break;
@@ -252,6 +265,9 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
       case FT_SYMBOLS:
         pszfieldtype = "symbols";
         break;
+      case FT_POLICYNAME:
+        pszfieldtype = "policyname";
+        break;
       // Time fields
       case FT_CTIME:
         pszfieldtype = "create_time";
@@ -279,6 +295,9 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
         pszfieldtype = "password_policy";
         break;
       // Other!
+      case FT_PASSWORDLEN:
+        pszfieldtype = "password_length";
+        break;
       case FT_UNKNOWNFIELDS:
         pszfieldtype = "unknownfields";
         break;
@@ -451,7 +470,7 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
 }
 
 struct XMLFilterWriterToString {
-  XMLFilterWriterToString(ostream &os, bool bWithFormatting) :
+  XMLFilterWriterToString(coStringXStream &os, bool bWithFormatting) :
   m_os(os), m_bWithFormatting(bWithFormatting)
   {}
   // operator
@@ -462,7 +481,7 @@ struct XMLFilterWriterToString {
   }
 private:
   XMLFilterWriterToString& operator=(const XMLFilterWriterToString&); // Do not implement
-  ostream &m_os;
+  coStringXStream &m_os;
   bool m_bWithFormatting;
 };
 
@@ -470,33 +489,33 @@ int PWSFilters::WriteFilterXMLFile(const StringX &filename,
                                    const PWSfile::HeaderRecord hdr,
                                    const StringX &currentfile)
 {
-#ifdef UNICODE
-  CUTF8Conv conv;
-  size_t fnamelen;
-  const unsigned char *fname = NULL;
-  conv.ToUTF8(filename, fname, fnamelen); 
-#else
-  const char *fname = filename.c_str();
-#endif
-  ofstream of(reinterpret_cast<const char *>(fname));
-  if (!of)
+  FILE *xmlfile = pws_os::FOpen(filename.c_str(), _T("wt"));
+  if (xmlfile == NULL)
     return PWScore::CANT_OPEN_FILE;
-  else
-    return WriteFilterXMLFile(of, hdr, currentfile, true);
+
+  coStringXStream oss;
+  int irc = WriteFilterXMLFile(oss, hdr, currentfile, true);
+
+  // Write it out to the file, clear string stream, close file
+  fwrite(oss.str().c_str(), 1, oss.str().length(), xmlfile);
+  oss.str("");
+  fclose(xmlfile);
+
+  return irc;
 }
 
-int PWSFilters::WriteFilterXMLFile(ostream &os,
+int PWSFilters::WriteFilterXMLFile(coStringXStream &oss,
                                    const PWSfile::HeaderRecord hdr,
                                    const StringX &currentfile,
                                    const bool bWithFormatting)
 {
   string str_hdr = GetFilterXMLHeader(currentfile, hdr);
-  os << str_hdr;
+  oss << str_hdr;
 
-  XMLFilterWriterToString put_filterxml(os, bWithFormatting);
+  XMLFilterWriterToString put_filterxml(oss, bWithFormatting);
   for_each(this->begin(), this->end(), put_filterxml);
 
-  os << "</filters>";
+  oss << "</filters>";
 
   return PWScore::SUCCESS;
 }
@@ -514,7 +533,7 @@ std::string PWSFilters::GetFilterXMLHeader(const StringX &currentfile,
   time_t time_now;
 
   time(&time_now);
-  const StringX now = PWSUtil::ConvertToDateTimeString(time_now, TMC_XML);
+  const StringX now = PWSUtil::ConvertToDateTimeString(time_now, PWSUtil::TMC_XML);
 
   oss << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
   oss << endl;
@@ -558,14 +577,14 @@ std::string PWSFilters::GetFilterXMLHeader(const StringX &currentfile,
     }
     if (hdr.m_whenlastsaved != 0) {
       StringX wls = PWSUtil::ConvertToDateTimeString(hdr.m_whenlastsaved,
-                                                     TMC_XML);
+                                                     PWSUtil::TMC_XML);
       utf8conv.ToUTF8(wls.c_str(), utf8, utf8Len);
       oss << "WhenLastSaved=\"";
       oss << reinterpret_cast<const char *>(utf8);
       oss << "\"" << endl;
     }
 
-    CUUIDGen huuid(hdr.m_file_uuid_array, true); // true to print canonically
+    CUUID huuid(*hdr.m_file_uuid.GetARep(), true); // true to print canonically
 
     oss << "Database_uuid=\"" << huuid << "\"" << endl;
   }
@@ -727,6 +746,7 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
       cs_criteria = cs_rule;
       break;
     case PWSMatch::MT_DCA:
+    case PWSMatch::MT_SHIFTDCA:
       Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
       break;
     case PWSMatch::MT_ENTRYTYPE:
