@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,7 +17,6 @@
 #include <msxml6.h>
 
 #include "../../util.h"
-#include "../../UUIDGen.h"
 #include "../../core.h"
 #include "../../PWSfileV3.h"
 #include "../../PWSFilters.h"
@@ -148,7 +147,7 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ErrorHandler::ignorableWarning(struct ISAXL
 MFilterSAX2ContentHandler::MFilterSAX2ContentHandler()
 {
   m_refCnt = 0;
-  m_strElemContent.clear();
+  m_sxElemContent = _T("");
   m_pSchema_Version = NULL;
   m_iXMLVersion = -1;
   m_iSchemaVersion = -1;
@@ -331,7 +330,7 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::startElement(
     }
   }
 
-  m_strElemContent = _T("");
+  m_sxElemContent = _T("");
 
   return S_OK;
 }
@@ -362,7 +361,7 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::characters(
 #endif
 
   szData[cchChars]=0;
-  m_strElemContent += szData;
+  m_sxElemContent += szData;
 
   delete [] szData;
   szData = NULL;
@@ -519,6 +518,12 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
     cur_filterentry->ftype = FT_DCA;
   }
 
+  else if (_tcscmp(szCurElement, _T("ShiftDCA")) == 0) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_SHIFTDCA;
+    cur_filterentry->ftype = FT_SHIFTDCA;
+  }
+
   else if (_tcscmp(szCurElement, _T("email")) == 0) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_STRING;
@@ -538,6 +543,12 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
     cur_filterentry->fstring = PWSUtil::DeDupString(cur_filterentry->fstring);
   }
 
+  else if (_tcscmp(szCurElement, _T("policyname")) == 0) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_STRING;
+    cur_filterentry->ftype = FT_POLICYNAME;
+  }
+
   else if (_tcscmp(szCurElement, _T("create_time")) == 0) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_DATE;
@@ -574,6 +585,12 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
     cur_filterentry->ftype = FT_XTIME_INT;
   }
 
+  else if (_tcscmp(szCurElement, _T("password_length")) == 0) {
+    m_type = DFTYPE_MAIN;
+    cur_filterentry->mtype = PWSMatch::MT_INTEGER;
+    cur_filterentry->ftype = FT_PASSWORDLEN;
+  }
+
   else if (_tcscmp(szCurElement, _T("entrytype")) == 0) {
     m_type = DFTYPE_MAIN;
     cur_filterentry->mtype = PWSMatch::MT_ENTRYTYPE;
@@ -700,44 +717,44 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
   }
 
   else if (_tcscmp(szCurElement, _T("rule")) == 0) {
-    ToUpper(m_strElemContent);
-    cur_filterentry->rule = PWSMatch::GetRule(m_strElemContent);
+    ToUpper(m_sxElemContent);
+    cur_filterentry->rule = PWSMatch::GetRule(m_sxElemContent);
   }
 
   else if (_tcscmp(szCurElement, _T("logic")) == 0) {
-    if (m_strElemContent == _T("or"))
+    if (m_sxElemContent == _T("or"))
       cur_filterentry->ltype = LC_OR;
     else
       cur_filterentry->ltype = LC_AND;
   }
 
   else if (_tcscmp(szCurElement, _T("string")) == 0) {
-    cur_filterentry->fstring = m_strElemContent;
+    cur_filterentry->fstring = m_sxElemContent;
   }
 
   else if (_tcscmp(szCurElement, _T("case")) == 0) {
-    cur_filterentry->fcase = _ttoi(m_strElemContent.c_str()) != 0;
+    cur_filterentry->fcase = _ttoi(m_sxElemContent.c_str()) != 0;
   }
 
   else if (_tcscmp(szCurElement, _T("warn")) == 0) {
-    cur_filterentry->fnum1 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum1 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (_tcscmp(szCurElement, _T("num1")) == 0) {
-    cur_filterentry->fnum1 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum1 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (_tcscmp(szCurElement, _T("num2")) == 0) {
-    cur_filterentry->fnum2 = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->fnum2 = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (_tcscmp(szCurElement, _T("unit")) == 0) {
-    cur_filterentry->funit = _ttoi(m_strElemContent.c_str());
+    cur_filterentry->funit = _ttoi(m_sxElemContent.c_str());
   }
 
   else if (_tcscmp(szCurElement, _T("date1")) == 0) {
     time_t t(0);
-    if (VerifyXMLDateString(m_strElemContent.c_str(), t) &&
+    if (VerifyXMLDateString(m_sxElemContent.c_str(), t) &&
         (t != (time_t)-1))
       cur_filterentry->fdate1 = t;
     else
@@ -746,7 +763,7 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
 
   else if (_tcscmp(szCurElement, _T("date2")) == 0) {
     time_t t(0);
-    if (VerifyXMLDateString(m_strElemContent.c_str(), t) &&
+    if (VerifyXMLDateString(m_sxElemContent.c_str(), t) &&
         (t != (time_t)-1))
       cur_filterentry->fdate2 = t;
     else
@@ -754,30 +771,34 @@ HRESULT STDMETHODCALLTYPE MFilterSAX2ContentHandler::endElement (
   }
 
   else if (_tcscmp(szCurElement, _T("dca")) == 0) {
-    cur_filterentry->fdca = (short)_ttoi(m_strElemContent.c_str());
+    cur_filterentry->fdca = (short)_ttoi(m_sxElemContent.c_str());
+  }
+
+  else if (_tcscmp(szCurElement, _T("shiftdca")) == 0) {
+    cur_filterentry->fdca = (short)_ttoi(m_sxElemContent.c_str());
   }
 
   else if (_tcscmp(szCurElement, _T("type")) == 0) {
-    if (m_strElemContent == _T("normal"))
+    if (m_sxElemContent == _T("normal"))
       cur_filterentry->etype = CItemData::ET_NORMAL;
-    else if (m_strElemContent == _T("alias"))
+    else if (m_sxElemContent == _T("alias"))
       cur_filterentry->etype = CItemData::ET_ALIAS;
-    else if (m_strElemContent == _T("shortcut"))
+    else if (m_sxElemContent == _T("shortcut"))
       cur_filterentry->etype = CItemData::ET_SHORTCUT;
-    else if (m_strElemContent == _T("aliasbase"))
+    else if (m_sxElemContent == _T("aliasbase"))
       cur_filterentry->etype = CItemData::ET_ALIASBASE;
-    else if (m_strElemContent == _T("shortcutbase"))
+    else if (m_sxElemContent == _T("shortcutbase"))
       cur_filterentry->etype = CItemData::ET_SHORTCUTBASE;
     else
       cur_filterentry->etype = CItemData::ET_INVALID;
   }
 
   else if (_tcscmp(szCurElement, _T("status")) == 0) {
-    if (m_strElemContent == _T("clean"))
+    if (m_sxElemContent == _T("clean"))
       cur_filterentry->estatus = CItemData::ES_CLEAN;
-    else if (m_strElemContent == _T("added"))
+    else if (m_sxElemContent == _T("added"))
       cur_filterentry->estatus = CItemData::ES_ADDED;
-    else if (m_strElemContent == _T("modified"))
+    else if (m_sxElemContent == _T("modified"))
       cur_filterentry->estatus = CItemData::ES_MODIFIED;
     else
       cur_filterentry->estatus = CItemData::ES_INVALID;
