@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -22,10 +22,13 @@
 #include "../PWSfileV3.h"
 #include "../VerifyFormat.h"
 #include "../Command.h"
+
 #include "os/UUID.h"
 
 #include <algorithm>
 
+extern const TCHAR *FORMATIMPORTED;
+
 using namespace std;
 using pws_os::CUUID;
 
@@ -33,10 +36,8 @@ XMLFileHandlers::XMLFileHandlers()
 {
   cur_entry = NULL;
   cur_pwhistory_entry = NULL;
-  m_strElemContent.clear();
+  m_sxElemContent = _T("");
 
-  m_sDefaultAutotypeString = _T("");
-  m_sDefaultUsername = _T("");
   m_delimiter = _T('\0');
   m_strErrorMessage = _T("");
 
@@ -48,11 +49,16 @@ XMLFileHandlers::XMLFileHandlers()
   m_bErrors = false;
 
   m_nITER = MIN_HASH_ITERATIONS;
+  
+  m_sxXML_DateTime = PWSUtil::GetTimeStamp(true).c_str();
 
-  // Following are user preferences stored in the database
-  for (int i = 0; i < NUMPREFSINXML; i++) {
-    prefsinXML[i] = -1;
-  }
+  // Set up copy of preferences to use for password policies and if we import into an
+  // empty database - Note: Load of empty string sets it to all defaults
+  PWSprefs::GetInstance()->Load(_T(""), true);
+  
+  // Get current DB default policy (any import DB default policy will be set after all
+  // the preferences are read in)
+  currentDB_default_pwp = PWSprefs::GetInstance()->GetDefaultPolicy();
 }
 
 XMLFileHandlers::~XMLFileHandlers()
@@ -85,6 +91,8 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
       m_numEntriesSkipped = 0;
       m_numEntriesRenamed = 0;
       m_numEntriesPWHErrors = 0;
+      m_numNoPolicies = 0;
+      m_numRenamedPolicies = 0;
       m_bEntryBeingProcessed = false;
       break;
     case XLE_ENTRY:
@@ -116,10 +124,10 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
       cur_entry->shiftdca = _T("");
       cur_entry->email = _T("");
       cur_entry->symbols = _T("");
+      cur_entry->policyname = _T("");
       cur_entry->ucprotected = 0;
       cur_entry->entrytype = NORMAL;
       cur_entry->bforce_normal_entry = false;
-      m_whichtime = -1;
       break;
     case XLE_HISTORY_ENTRY:
       if (m_bValidation)
@@ -135,16 +143,25 @@ bool XMLFileHandlers::ProcessStartElement(const int icurrent_element)
     case XLE_XTIMEX:
     case XLE_PMTIMEX:
     case XLE_RMTIMEX:
+    case XLE_CHANGEDX:
       break;
-    case XLE_CTIME:
-    case XLE_ATIME:
-    case XLE_LTIME:
-    case XLE_XTIME:
-    case XLE_PMTIME:
-    case XLE_RMTIME:
-    case XLE_CHANGED:
-      m_whichtime = icurrent_element;
+    case XLE_PASSWORDPOLICYNAMES:
+      m_bInPolicyNames = true;
       break;
+    case XLE_POLICY:
+      m_bPolicyBeingProcessed = true;
+      if (m_bValidation)
+        return false;
+
+      // Clear all fields
+      m_Named_pwp.Empty();
+      m_PolicyName = _T("");
+
+      break;
+    case XLE_EMPTYGROUPS:
+      m_bInEmptyGroups = true;
+      break;
+    case XLE_PWNAME:
     default:
       break;
   }
@@ -156,9 +173,13 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
   StringX buffer(_T(""));
   int i;
 
+  PWSprefs::BoolPrefs bpref(PWSprefs::NumBoolPrefs);
+  PWSprefs::IntPrefs ipref(PWSprefs::NumIntPrefs);
+  PWSprefs::StringPrefs spref(PWSprefs::NumStringPrefs);
+
   switch (icurrent_element) {
     case XLE_NUMBERHASHITERATIONS:
-      i = _ttoi(m_strElemContent.c_str());
+      i = _ttoi(m_sxElemContent.c_str());
       if (i > MIN_HASH_ITERATIONS) {
         m_nITER = i;
       }
@@ -167,154 +188,270 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
       m_ventries.push_back(cur_entry);
       m_numEntries++;
       break;
-    case XLE_DISPLAYEXPANDEDADDEDITDLG:
-      // Obsoleted in 3.18
+
+    case XLE_PREFERENCES:
+      // Preferences finished - get the default policy (if any) from the import XML file
+      importDB_default_pwp = PWSprefs::GetInstance()->GetDefaultPolicy(true);
+      break;
+    // Boolean DB preferences
+    case XLE_PREF_SHOWPWDEFAULT:
+      bpref = PWSprefs::ShowPWDefault;
+      break;
+    case XLE_PREF_SHOWPASSWORDINTREE:
+      bpref = PWSprefs::ShowPasswordInTree;
+      break;
+    case XLE_PREF_SORTASCENDING:
+      bpref = PWSprefs::SortAscending;
+      break;
+    case XLE_PREF_USEDEFAULTUSER:
+      bpref = PWSprefs::UseDefaultUser;
+      break;
+    case XLE_PREF_SAVEIMMEDIATELY:
+      bpref = PWSprefs::SaveImmediately;
+      break;
+    case XLE_PREF_PWUSELOWERCASE:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseLowercase;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseLowercase;
+      else
+        bpref = PWSprefs::PWUseLowercase;
+      break;
+    case XLE_PREF_PWUSEUPPERCASE:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseUppercase;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseUppercase;
+      else
+        bpref = PWSprefs::PWUseUppercase;
+      break;
+    case XLE_PREF_PWUSEDIGITS:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseDigits;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseDigits;
+      else
+        bpref = PWSprefs::PWUseDigits;
+      break;
+    case XLE_PREF_PWUSESYMBOLS:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseSymbols;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseSymbols;
+      else
+        bpref = PWSprefs::PWUseSymbols;
+      break;
+    case XLE_PREF_PWUSEHEXDIGITS:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseHexDigits;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseHexDigits;
+      else
+        bpref = PWSprefs::PWUseHexDigits;
+      break;
+    case XLE_PREF_PWUSEEASYVISION:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::UseEasyVision;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::UseEasyVision;
+      else
+        bpref = PWSprefs::PWUseEasyVision;
+      break;
+    case XLE_PREF_MAINTAINDATETIMESTAMPS:
+      bpref = PWSprefs::MaintainDateTimeStamps;
+      break;
+    case XLE_PREF_SAVEPASSWORDHISTORY:
+      bpref = PWSprefs::SavePasswordHistory;
+      break;
+    case XLE_PREF_SHOWNOTESDEFAULT:
+      bpref = PWSprefs::ShowNotesDefault;
+      break;
+    case XLE_PREF_SHOWUSERNAMEINTREE:
+      bpref = PWSprefs::ShowUsernameInTree;
+      break;
+    case XLE_PREF_PWMAKEPRONOUNCEABLE:
+      if (m_bPolicyBeingProcessed)
+        if (m_sxElemContent == _T("1"))
+          m_Named_pwp.flags |= PWPolicy::MakePronounceable;
+        else
+          m_Named_pwp.flags &= ~PWPolicy::MakePronounceable;
+      else
+        bpref = PWSprefs::PWMakePronounceable;
+      break;
+    case XLE_PREF_LOCKDBONIDLETIMEOUT:
+      bpref = PWSprefs::LockDBOnIdleTimeout;
+      break;
+    case XLE_PREF_COPYPASSWORDWHENBROWSETOURL:
+      bpref = PWSprefs::CopyPasswordWhenBrowseToURL;
+      break;
+    // Integer DB preferences
+    case XLE_PREF_PWDEFAULTLENGTH:
+      if (m_bPolicyBeingProcessed)
+        m_Named_pwp.length = _ttoi(m_sxElemContent.c_str());
+      else
+        ipref = PWSprefs::PWDefaultLength;
+      break;
+    case XLE_PREF_IDLETIMEOUT:
+      ipref = PWSprefs::IdleTimeout;
+      break;
+    case XLE_PREF_TREEDISPLAYSTATUSATOPEN:
+      // Since value is a string - need to convert here to corresponding integer value
+      if (m_sxElemContent == _T("AllCollapsed"))
+        i = PWSprefs::AllCollapsed;
+      else if (m_sxElemContent == _T("AllExpanded"))
+        i = PWSprefs::AllExpanded;
+      else if (m_sxElemContent == _T("AsPerLastSave"))
+        i = PWSprefs::AsPerLastSave;
+      else
+        break;
+      PWSprefs::GetInstance()->SetPref(PWSprefs::TreeDisplayStatusAtOpen, i, true);
       return;
-    case XLE_LOCKDBONIDLETIMEOUT:
-    case XLE_IDLETIMEOUT:
-    case XLE_MAINTAINDATETIMESTAMPS:
-    case XLE_NUMPWHISTORYDEFAULT:
-    case XLE_PWDEFAULTLENGTH:
-    case XLE_PWDIGITMINLENGTH:
-    case XLE_PWLOWERCASEMINLENGTH:
-    case XLE_PWMAKEPRONOUNCEABLE:
-    case XLE_PWSYMBOLMINLENGTH:
-    case XLE_PWUPPERCASEMINLENGTH:
-    case XLE_PWUSEDIGITS:
-    case XLE_PWUSEEASYVISION:
-    case XLE_PWUSEHEXDIGITS:
-    case XLE_PWUSELOWERCASE:
-    case XLE_PWUSESYMBOLS:
-    case XLE_PWUSEUPPERCASE:
-    case XLE_SAVEIMMEDIATELY:
-    case XLE_SAVEPASSWORDHISTORY:
-    case XLE_SHOWNOTESDEFAULT:
-    case XLE_SHOWPASSWORDINTREE:
-    case XLE_SHOWPWDEFAULT:
-    case XLE_SHOWUSERNAMEINTREE:
-    case XLE_SORTASCENDING:
-    case XLE_USEDEFAULTUSER:
-      prefsinXML[icurrent_element - XLE_PREF_START] = _ttoi(m_strElemContent.c_str());
+    case XLE_PREF_NUMPWHISTORYDEFAULT:
+      ipref = PWSprefs::NumPWHistoryDefault;
       break;
-    case XLE_TREEDISPLAYSTATUSATOPEN:
-      if (m_strElemContent == _T("AllCollapsed"))
-        prefsinXML[XLE_TREEDISPLAYSTATUSATOPEN - XLE_PREF_START] = PWSprefs::AllCollapsed;
-      else if (m_strElemContent == _T("AllExpanded"))
-        prefsinXML[XLE_TREEDISPLAYSTATUSATOPEN - XLE_PREF_START] = PWSprefs::AllExpanded;
-      else if (m_strElemContent == _T("AsPerLastSave"))
-        prefsinXML[XLE_TREEDISPLAYSTATUSATOPEN - XLE_PREF_START] = PWSprefs::AsPerLastSave;
+    case XLE_PREF_PWDIGITMINLENGTH:
+      if (m_bPolicyBeingProcessed)
+        m_Named_pwp.digitminlength = _ttoi(m_sxElemContent.c_str());
+      else
+        ipref = PWSprefs::PWDigitMinLength;
       break;
-    case XLE_DEFAULTUSERNAME:
-      m_sDefaultUsername = m_strElemContent.c_str();
+    case XLE_PREF_PWLOWERCASEMINLENGTH:
+      if (m_bPolicyBeingProcessed)
+         m_Named_pwp.lowerminlength = _ttoi(m_sxElemContent.c_str());
+      else
+        ipref = PWSprefs::PWLowercaseMinLength;
       break;
-    case XLE_DEFAULTAUTOTYPESTRING:
-      m_sDefaultAutotypeString = m_strElemContent.c_str();
+    case XLE_PREF_PWSYMBOLMINLENGTH:
+      if (m_bPolicyBeingProcessed)
+        m_Named_pwp.symbolminlength = _ttoi(m_sxElemContent.c_str());
+      else
+        ipref = PWSprefs::PWSymbolMinLength;
       break;
+    case XLE_PREF_PWUPPERCASEMINLENGTH:
+      if (m_bPolicyBeingProcessed)
+        m_Named_pwp.upperminlength = _ttoi(m_sxElemContent.c_str());
+      else
+        ipref = PWSprefs::PWUppercaseMinLength;
+      break;
+    // String DB preferences
+    case XLE_PREF_DEFAULTUSERNAME:
+      spref = PWSprefs::DefaultUsername;
+      break;
+    case XLE_PREF_DEFAULTAUTOTYPESTRING:
+      spref = PWSprefs::DefaultAutotypeString;
+      break;
+    case XLE_PREF_DEFAULTSYMBOLS:
+      spref = PWSprefs::DefaultSymbols;
+      break;
+
+    case XLE_PASSWORDPOLICYNAMES:
+      m_bInPolicyNames = false;
+      break;
+    case XLE_EMPTYGROUPS:
+      m_bInEmptyGroups = false;
+      break;
+    case XLE_EGNAME:
+      if (!m_sxElemContent.empty() &&
+          find(m_vEmptyGroups.begin(), m_vEmptyGroups.end(), m_sxElemContent) == m_vEmptyGroups.end())
+        m_vEmptyGroups.push_back(m_sxElemContent);
+      break;
+
     // MUST be in the same order as enum beginning STR_GROUP...
     case XLE_GROUP:
-      cur_entry->group = m_strElemContent;
+      cur_entry->group = m_sxElemContent;
       break;
     case XLE_TITLE:
-      cur_entry->title = m_strElemContent;
+      cur_entry->title = m_sxElemContent;
       break;
     case XLE_USERNAME:
-      cur_entry->username = m_strElemContent;
+      cur_entry->username = m_sxElemContent;
       break;
     case XLE_URL:
-      cur_entry->url = m_strElemContent;
+      cur_entry->url = m_sxElemContent;
       break;
     case XLE_AUTOTYPE:
-      cur_entry->autotype = m_strElemContent;
+      cur_entry->autotype = m_sxElemContent;
       break;
     case XLE_NOTES:
-      cur_entry->notes = m_strElemContent;
+      cur_entry->notes = m_sxElemContent;
       break;
     case XLE_UUID:
-      cur_entry->uuid = m_strElemContent;
+      cur_entry->uuid = m_sxElemContent;
       break;
     case XLE_PASSWORD:
-      cur_entry->password = m_strElemContent;
-      if (Replace(m_strElemContent, _T(':'), _T(';')) <= 2) {
-        if (m_strElemContent.substr(0, 2) == _T("[[") &&
-            m_strElemContent.substr(m_strElemContent.length() - 2) == _T("]]")) {
+      cur_entry->password = m_sxElemContent;
+      if (Replace(m_sxElemContent, _T(':'), _T(';')) <= 2) {
+        if (m_sxElemContent.substr(0, 2) == _T("[[") &&
+            m_sxElemContent.substr(m_sxElemContent.length() - 2) == _T("]]")) {
             cur_entry->entrytype = ALIAS;
         }
-        if (m_strElemContent.substr(0, 2) == _T("[~") &&
-            m_strElemContent.substr(m_strElemContent.length() - 2) == _T("~]")) {
+        if (m_sxElemContent.substr(0, 2) == _T("[~") &&
+            m_sxElemContent.substr(m_sxElemContent.length() - 2) == _T("~]")) {
             cur_entry->entrytype = SHORTCUT;
         }
       }
       break;
     case XLE_CTIMEX:
-      cur_entry->ctime = m_strElemContent;
+      cur_entry->ctime = m_sxElemContent;
       break;
     case XLE_ATIMEX:
-      cur_entry->atime = m_strElemContent;
+      cur_entry->atime = m_sxElemContent;
       break;
     case XLE_XTIMEX:
-      cur_entry->xtime = m_strElemContent;
+      cur_entry->xtime = m_sxElemContent;
       break;
     case XLE_PMTIMEX:
-      cur_entry->pmtime = m_strElemContent;
+      cur_entry->pmtime = m_sxElemContent;
       break;
     case XLE_RMTIMEX:
-      cur_entry->rmtime = m_strElemContent;
-      break;
-    case XLE_CTIME:
-      Replace(cur_entry->ctime, _T('-'), _T('/'));
-      m_whichtime = -1;
-      break;
-    case XLE_ATIME:
-      Replace(cur_entry->atime, _T('-'), _T('/'));
-      m_whichtime = -1;
-      break;
-    case XLE_LTIME:
-    case XLE_XTIME:
-      Replace(cur_entry->xtime, _T('-'), _T('/'));
-      m_whichtime = -1;
-      break;
-    case XLE_PMTIME:
-      Replace(cur_entry->pmtime, _T('-'), _T('/'));
-      m_whichtime = -1;
-      break;
-    case XLE_RMTIME:
-      Replace(cur_entry->rmtime, _T('-'), _T('/'));
-      m_whichtime = -1;
+      cur_entry->rmtime = m_sxElemContent;
       break;
     case XLE_XTIME_INTERVAL:
-      cur_entry->xtime_interval = Trim(m_strElemContent);
+      cur_entry->xtime_interval = Trim(m_sxElemContent);
       break;
     case XLE_RUNCOMMAND:
-      cur_entry->run_command = m_strElemContent;
+      cur_entry->run_command = m_sxElemContent;
       break;
     case XLE_DCA:
-      cur_entry->dca = Trim(m_strElemContent);
+      cur_entry->dca = Trim(m_sxElemContent);
       break;
     case XLE_SHIFTDCA:
-      cur_entry->shiftdca = Trim(m_strElemContent);
+      cur_entry->shiftdca = Trim(m_sxElemContent);
       break;
     case XLE_EMAIL:
-      cur_entry->email = m_strElemContent;
+      cur_entry->email = m_sxElemContent;
       break;
     case XLE_PROTECTED:
-      if (m_strElemContent == _T("1"))
+      if (m_sxElemContent == _T("1"))
         cur_entry->ucprotected = 1;
       break;
     case XLE_SYMBOLS:
-      cur_entry->symbols = m_strElemContent;
+      if (m_bPolicyBeingProcessed)
+        m_Named_pwp.symbols = m_sxElemContent;
+      else
+        cur_entry->symbols = m_sxElemContent;
+      break;
+    case XLE_ENTRY_PASSWORDPOLICYNAME:
+      cur_entry->policyname = m_sxElemContent;
       break;
     case XLE_STATUS:
-      i = _ttoi(m_strElemContent.c_str());
+      i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%01x"), i);
       cur_entry->pwhistory = buffer;
       break;
     case XLE_MAX:
-      i = _ttoi(m_strElemContent.c_str());
+      i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%02x"), i);
       cur_entry->pwhistory += buffer;
       break;
     case XLE_NUM:
-      i = _ttoi(m_strElemContent.c_str());
+      i = _ttoi(m_sxElemContent.c_str());
       Format(buffer, _T("%02x"), i);
       cur_entry->pwhistory += buffer;
       break;
@@ -328,144 +465,136 @@ void XMLFileHandlers::ProcessEndElement(const int icurrent_element)
       delete cur_pwhistory_entry;
       cur_pwhistory_entry = NULL;
       break;
-    case XLE_CHANGED:
-      ASSERT(cur_pwhistory_entry != NULL);
-      Replace(cur_pwhistory_entry->changed, _T('-'), _T('/'));
-      Trim(cur_pwhistory_entry->changed);
-      if (cur_pwhistory_entry->changed.empty()) {
-        //                                 1234567890123456789
-        cur_pwhistory_entry->changed = _T("1970-01-01 00:00:00");
-      }
-      m_whichtime = -1;
-      break;
     case XLE_CHANGEDX:
-      cur_pwhistory_entry->changed = m_strElemContent;
+      cur_pwhistory_entry->changed = m_sxElemContent;
       break;
     case XLE_OLDPASSWORD:
       ASSERT(cur_pwhistory_entry != NULL);
-      cur_pwhistory_entry->oldpassword = m_strElemContent;
+      cur_pwhistory_entry->oldpassword = m_sxElemContent;
       break;
     case XLE_ENTRY_PWLENGTH:
-      cur_entry->pwp.length = _ttoi(m_strElemContent.c_str());
+      cur_entry->pwp.length = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWUSEDIGITS:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseDigits;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseDigits;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseDigits;
+        cur_entry->pwp.flags &= ~PWPolicy::UseDigits;
       break;
     case XLE_ENTRY_PWUSEEASYVISION:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseEasyVision;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseEasyVision;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseEasyVision;
+        cur_entry->pwp.flags &= ~PWPolicy::UseEasyVision;
       break;
     case XLE_ENTRY_PWUSEHEXDIGITS:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseHexDigits;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseHexDigits;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseHexDigits;
+        cur_entry->pwp.flags &= ~PWPolicy::UseHexDigits;
       break;
     case XLE_ENTRY_PWUSELOWERCASE:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseLowercase;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseLowercase;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseLowercase;
+        cur_entry->pwp.flags &= ~PWPolicy::UseLowercase;
       break;
     case XLE_ENTRY_PWUSESYMBOLS:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseSymbols;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseSymbols;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseSymbols;
+        cur_entry->pwp.flags &= ~PWPolicy::UseSymbols;
       break;
     case XLE_ENTRY_PWUSEUPPERCASE:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyUseUppercase;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::UseUppercase;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyUseUppercase;
+        cur_entry->pwp.flags &= ~PWPolicy::UseUppercase;
       break;
     case XLE_ENTRY_PWMAKEPRONOUNCEABLE:
-      if (m_strElemContent == _T("1"))
-        cur_entry->pwp.flags |= PWSprefs::PWPolicyMakePronounceable;
+      if (m_sxElemContent == _T("1"))
+        cur_entry->pwp.flags |= PWPolicy::MakePronounceable;
       else
-        cur_entry->pwp.flags &= ~PWSprefs::PWPolicyMakePronounceable;
+        cur_entry->pwp.flags &= ~PWPolicy::MakePronounceable;
       break;
     case XLE_ENTRY_PWDIGITMINLENGTH:
-      cur_entry->pwp.digitminlength = _ttoi(m_strElemContent.c_str());
+      cur_entry->pwp.digitminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWLOWERCASEMINLENGTH:
-      cur_entry->pwp.lowerminlength = _ttoi(m_strElemContent.c_str());
+      cur_entry->pwp.lowerminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWSYMBOLMINLENGTH:
-      cur_entry->pwp.symbolminlength = _ttoi(m_strElemContent.c_str());
+      cur_entry->pwp.symbolminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_ENTRY_PWUPPERCASEMINLENGTH:
-      cur_entry->pwp.upperminlength = _ttoi(m_strElemContent.c_str());
-      break;
-    case XLE_DATE:
-      switch (m_whichtime) {
-        case XLE_CTIME:
-          cur_entry->ctime = m_strElemContent;
-          break;
-        case XLE_ATIME:
-          cur_entry->atime = m_strElemContent;
-          break;
-        case XLE_LTIME:
-        case XLE_XTIME:
-          cur_entry->xtime = m_strElemContent;
-          break;
-        case XLE_PMTIME:
-          cur_entry->pmtime = m_strElemContent;
-          break;
-        case XLE_RMTIME:
-          cur_entry->rmtime = m_strElemContent;
-          break;
-        case XLE_CHANGED:
-          ASSERT(cur_pwhistory_entry != NULL);
-          cur_pwhistory_entry->changed = m_strElemContent;
-          break;
-        default:
-          ASSERT(0);
-      }
-      break;
-    case XLE_TIME:
-      switch (m_whichtime) {
-        case XLE_CTIME:
-          cur_entry->ctime += _T(" ") + m_strElemContent;
-          break;
-        case XLE_ATIME:
-          cur_entry->atime += _T(" ") + m_strElemContent;
-          break;
-        case XLE_LTIME:
-        case XLE_XTIME:
-          cur_entry->xtime += _T(" ") + m_strElemContent;
-          break;
-        case XLE_PMTIME:
-          cur_entry->pmtime += _T(" ") + m_strElemContent;
-          break;
-        case XLE_RMTIME:
-          cur_entry->rmtime += _T(" ") + m_strElemContent;
-          break;
-        case XLE_CHANGED:
-          ASSERT(cur_pwhistory_entry != NULL);
-          cur_pwhistory_entry->changed += _T(" ") + m_strElemContent;
-          break;
-        default:
-          ASSERT(0);
-      }
+      cur_entry->pwp.upperminlength = _ttoi(m_sxElemContent.c_str());
       break;
     case XLE_PASSWORDSAFE:
-    case XLE_PREFERENCES:
     case XLE_PWHISTORY:
     case XLE_HISTORY_ENTRIES:
     case XLE_ENTRY_PASSWORDPOLICY:
     default:
       break;
+    case XLE_POLICY:
+    {
+      // Deal with Named Policies in the XML file
+      PWPolicy currentDB_st_pp;
+      if (m_pXMLcore->GetPolicyFromName(m_PolicyName, currentDB_st_pp)) {
+        // It already exists in current database
+        if (currentDB_st_pp != m_Named_pwp) {
+          // They are not the same
+          m_pXMLcore->MakePolicyUnique(m_mapRenamedPolicies, m_PolicyName,
+                              m_sxXML_DateTime, IDSC_IMPORTPOLICY);
+          // Now renamed add it
+          m_MapPSWDPLC[m_PolicyName] = m_Named_pwp;
+          m_numRenamedPolicies++;
+        }
+      } else {
+        // Doesn't exist - need to add it
+        m_MapPSWDPLC[m_PolicyName] = m_Named_pwp;
+      }
+      m_bPolicyBeingProcessed = false;
+      break;
+    }
+    case XLE_PWNAME:
+      m_PolicyName = m_sxElemContent.c_str();
+      break;
   }
+  
+  // If we have processed a DB preference - add it to our copy
+  if (bpref != PWSprefs::NumBoolPrefs)    // boolean
+    PWSprefs::GetInstance()->SetPref(bpref, _ttoi(m_sxElemContent.c_str()) == 0 ? false : true, true);
+  if (ipref != PWSprefs::NumIntPrefs)     // integer
+    PWSprefs::GetInstance()->SetPref(ipref, _ttoi(m_sxElemContent.c_str()), true);
+  if (spref != PWSprefs::NumStringPrefs)  // string
+    PWSprefs::GetInstance()->SetPref(spref, m_sxElemContent, true);
 }
 
-void XMLFileHandlers::AddEntries()
+void XMLFileHandlers::AddXMLEntries()
 {
+  // First add any Policy Names imported that are not already in the database
+  // This must be done prior to importing entries that may reference them
+  if (!m_MapPSWDPLC.empty()) {
+    Command *pcmd = DBPolicyNamesCommand::Create(m_pXMLcore, m_MapPSWDPLC,
+                            DBPolicyNamesCommand::NP_ADDNEW);
+    m_pmulticmds->Add(pcmd);
+  }
+  
+  // Then add any Empty Groups imported that are not already in the database
+  if (!m_vEmptyGroups.empty()) {
+    Command *pcmd = DBEmptyGroupsCommand::Create(m_pXMLcore, m_vEmptyGroups,
+                           DBEmptyGroupsCommand::EG_ADDALL);
+    m_pmulticmds->Add(pcmd);
+  }
+
+  // Gte current DB default password policy and that from the XML file and
+  // check that they are the same?
+  PWPolicy st_to_default_pp, st_import_default_pp;
+  st_to_default_pp = PWSprefs::GetInstance()->GetDefaultPolicy();
+  st_import_default_pp = PWSprefs::GetInstance()->GetDefaultPolicy(true);
+  const bool bPWPDefaults_Different = st_to_default_pp != st_import_default_pp;
+        
+  StringX sxEntriesWithNewNamedPolicies;
   vdb_entries::iterator entry_iter;
   CItemData ci_temp;
   bool bMaintainDateTimeStamps = PWSprefs::GetInstance()->
@@ -484,8 +613,10 @@ void XMLFileHandlers::AddEntries()
   m_pmulticmds->Add(pcmd1);
 
   for (entry_iter = m_ventries.begin(); entry_iter != m_ventries.end(); entry_iter++) {
+    bool bNoPolicy(false);
     pw_entry *cur_entry = *entry_iter;
     StringX sxtitle(cur_entry->title);
+    StringX sxMissingPolicyName;
     EmptyIfOnlyWhiteSpace(sxtitle);
     // Title and Password are mandatory fields!
     if (sxtitle.empty() || cur_entry->password.empty()) {
@@ -584,7 +715,8 @@ void XMLFileHandlers::AddEntries()
     EmptyIfOnlyWhiteSpace(sxnewtitle);
 
     bool conflict = !m_pXMLcore->MakeEntryUnique(setGTU,
-                                                 sxnewgroup, sxnewtitle, cur_entry->username,
+                                                 sxnewgroup, sxnewtitle,
+                                                 cur_entry->username,
                                                  IDSC_IMPORTNUMBER);
 
     if (conflict) {
@@ -601,51 +733,119 @@ void XMLFileHandlers::AddEntries()
     }
 
     ci_temp.SetGroup(sxnewgroup);
+
     if (!sxnewtitle.empty())
       ci_temp.SetTitle(sxnewtitle, m_delimiter);
+
     EmptyIfOnlyWhiteSpace(cur_entry->username);
     if (!cur_entry->username.empty())
       ci_temp.SetUser(cur_entry->username);
+
     if (!cur_entry->password.empty())
       ci_temp.SetPassword(cur_entry->password);
+
     EmptyIfOnlyWhiteSpace(cur_entry->url);
     if (!cur_entry->url.empty())
       ci_temp.SetURL(cur_entry->url);
+
     EmptyIfOnlyWhiteSpace(cur_entry->autotype);
     if (!cur_entry->autotype.empty())
       ci_temp.SetAutoType(cur_entry->autotype);
+
     if (!cur_entry->ctime.empty())
       ci_temp.SetCTime(cur_entry->ctime.c_str());
+
     if (!cur_entry->pmtime.empty())
       ci_temp.SetPMTime(cur_entry->pmtime.c_str());
+
     if (!cur_entry->atime.empty())
       ci_temp.SetATime(cur_entry->atime.c_str());
+
     if (!cur_entry->xtime.empty())
       ci_temp.SetXTime(cur_entry->xtime.c_str());
+
     if (!cur_entry->xtime_interval.empty()) {
       int numdays = _ttoi(cur_entry->xtime_interval.c_str());
       if (numdays > 0 && numdays <= 3650)
         ci_temp.SetXTimeInt(numdays);
     }
+
     if (!cur_entry->rmtime.empty())
       ci_temp.SetRMTime(cur_entry->rmtime.c_str());
+
     if (!cur_entry->run_command.empty())
       ci_temp.SetRunCommand(cur_entry->run_command);
+
     if (cur_entry->pwp.flags != 0)
       ci_temp.SetPWPolicy(cur_entry->pwp);
+
     if (!cur_entry->dca.empty())
       ci_temp.SetDCA(cur_entry->dca.c_str());
+
     if (!cur_entry->shiftdca.empty())
       ci_temp.SetShiftDCA(cur_entry->shiftdca.c_str());
+
     if (!cur_entry->email.empty())
       ci_temp.SetEmail(cur_entry->email);
+
     if (cur_entry->ucprotected)
       ci_temp.SetProtected(cur_entry->ucprotected != 0);
+
     if (!cur_entry->symbols.empty())
       ci_temp.SetSymbols(cur_entry->symbols);
 
+    if (cur_entry->policyname.empty()) {
+      // Not using a named password policy
+      if (cur_entry->pwp.flags == 0) {
+        // If no specific policy (meaning use default) and they are different,
+        // Make this entry have the imported default its specific policy
+        if (bPWPDefaults_Different) {
+          ci_temp.SetPWPolicy(st_import_default_pp);
+        }
+      } else {
+        // Has been imported with a specific password policy - set it
+        ci_temp.SetPWPolicy(cur_entry->pwp);
+      }
+    } else {
+      // Using a named password policy
+      // Checks:
+      // 1. Are we about to add it?
+      // 2. If not, did we rename it?
+      // 3. Is it in our current DB?
+
+      if (m_MapPSWDPLC.find(cur_entry->policyname) == m_MapPSWDPLC.end()) {
+        // We are not about to add it - so
+        // Is it one we renamed because it exists in the current
+        // database but with different settings?
+        std::map<StringX, StringX>::const_iterator citer;
+        citer = m_mapRenamedPolicies.find(cur_entry->policyname);
+        if (citer != m_mapRenamedPolicies.end()) {
+          // Yes we did, so use renamed version
+          cur_entry->policyname = citer->second;
+          StringX sxChanged = L"\r\n\xab" + cur_entry->group    + L"\xbb " +
+	                            L"\xab"     + cur_entry->title    + L"\xbb " +
+	                            L"\xab"     + cur_entry->username + L"\xbb";
+          sxEntriesWithNewNamedPolicies += sxChanged;
+        } else {
+          // No we didn't, verify current database has it
+          PWPolicy currentDB_named_st_pp;
+          if (!m_pXMLcore->GetPolicyFromName(cur_entry->policyname, currentDB_named_st_pp)) {
+            // Not here - make a note and clear the name
+            // As we have no information about it's settings we can't even give
+            // this entry a specific policy
+            sxMissingPolicyName = cur_entry->policyname;
+            cur_entry->policyname = _T("");
+            m_numNoPolicies++;
+            bNoPolicy = true;
+          }
+        }
+      }
+      ci_temp.SetPolicyName(cur_entry->policyname);
+    }
+
     StringX newPWHistory;
     stringT strPWHErrorList;
+
     switch (VerifyXMLImportPWHistoryString(cur_entry->pwhistory,
                                            newPWHistory, strPWHErrorList)) {
       case PWH_OK:
@@ -690,6 +890,16 @@ void XMLFileHandlers::AddEntries()
       ci_temp.SetStatus(CItemData::ES_ADDED);
     }
 
+    StringX sx_imported;
+    Format(sx_imported, FORMATIMPORTED,
+                        cur_entry->group.c_str(), cur_entry->title.c_str(), cur_entry->username.c_str());
+    m_prpt->WriteLine(sx_imported.c_str());
+
+    if (bNoPolicy) {
+      Format(sx_imported, IDSC_MISSINGPOLICYNAME, sxMissingPolicyName.c_str());
+      m_prpt->WriteLine(sx_imported.c_str());
+    }
+
     m_pXMLcore->GUISetupDisplayInfo(ci_temp);
     Command *pcmd = AddEntryCommand::Create(m_pXMLcore, ci_temp);
     pcmd->SetNoGUINotify();
@@ -712,78 +922,21 @@ void XMLFileHandlers::AddEntries()
                                             UpdateGUICommand::WN_EXECUTE_REDO,
                                             UpdateGUICommand::GUI_REDO_IMPORT);
   m_pmulticmds->Add(pcmd2);
+  
+  if (!sxEntriesWithNewNamedPolicies.empty()) {
+    StringX sxRenamedPolicies;
+    Format(sxRenamedPolicies, IDSC_ENTRIES_POLICIES, m_sxXML_DateTime.c_str(),
+                sxEntriesWithNewNamedPolicies.c_str());
+    m_prpt->WriteLine();
+    m_prpt->WriteLine(sxRenamedPolicies.c_str());
+    m_prpt->WriteLine();
+  }
 }
 
 void XMLFileHandlers::AddDBPreferences()
 {
-  PWSprefs *prefs = PWSprefs::GetInstance();
-  int ivalue;
-  // Integer/Boolean preferences
-  /* Obsolete in 3.18
-  if ((ivalue = prefsinXML[XLE_DISPLAYEXPANDEDADDEDITDLG - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::DisplayExpandedAddEditDlg, ivalue == 1);
-  */
-  if ((ivalue = prefsinXML[XLE_LOCKDBONIDLETIMEOUT - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::LockDBOnIdleTimeout, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_IDLETIMEOUT - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::IdleTimeout, ivalue);
-  if ((ivalue = prefsinXML[XLE_MAINTAINDATETIMESTAMPS - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::MaintainDateTimeStamps, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_NUMPWHISTORYDEFAULT - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::NumPWHistoryDefault, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWDIGITMINLENGTH - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWDigitMinLength, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWLOWERCASEMINLENGTH - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWLowercaseMinLength, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWMAKEPRONOUNCEABLE - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWMakePronounceable, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWSYMBOLMINLENGTH - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWSymbolMinLength, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWUPPERCASEMINLENGTH - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUppercaseMinLength, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWDEFAULTLENGTH - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWDefaultLength, ivalue);
-  if ((ivalue = prefsinXML[XLE_PWUSEDIGITS - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseDigits, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWUSEEASYVISION - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseEasyVision, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWUSEHEXDIGITS - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseHexDigits, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWUSELOWERCASE - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseLowercase, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWUSESYMBOLS - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseSymbols, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_PWUSEUPPERCASE - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::PWUseUppercase, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SAVEIMMEDIATELY - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::SaveImmediately, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SAVEPASSWORDHISTORY - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::SavePasswordHistory, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SHOWNOTESDEFAULT - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::ShowNotesDefault, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SHOWPASSWORDINTREE - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::ShowPasswordInTree, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SHOWPWDEFAULT - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::ShowPWDefault, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SHOWUSERNAMEINTREE - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::ShowUsernameInTree, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_SORTASCENDING - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::SortAscending, ivalue == 1);
-  if ((ivalue = prefsinXML[XLE_TREEDISPLAYSTATUSATOPEN - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::TreeDisplayStatusAtOpen, ivalue);
-  if ((ivalue = prefsinXML[XLE_USEDEFAULTUSER - XLE_PREF_START]) != -1)
-    prefs->SetPref(PWSprefs::UseDefaultUser, ivalue == 1);
-
-  // String preferences
-  if (!m_sDefaultUsername.empty())
-    prefs->SetPref(PWSprefs::DefaultUsername,
-                   m_sDefaultUsername.c_str());
-  if (!m_sDefaultAutotypeString.empty())
-    prefs->SetPref(PWSprefs::DefaultAutotypeString,
-                   m_sDefaultAutotypeString.c_str());
-  if (!m_sDefaultSymbols.empty())
-    prefs->SetPref(PWSprefs::DefaultSymbols,
-                   m_sDefaultSymbols.c_str());
+  // Copy over preferences from XML input to this DB (only if DB was empty before import)
+  PWSprefs::GetInstance()->UpdateFromCopyPrefs(PWSprefs::ptDatabase);
 }
 
 #endif /* USE_XML_LIBRARY */
