@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -13,7 +13,7 @@
 #include "ThisMfcApp.h"    // For Help
 #include "GeneralMsgBox.h"
 #include "DboxMain.h"
-#include "PwFont.h"
+#include "Fonts.h"
 
 #include "AddEdit_Basic.h"
 #include "AddEdit_PropertySheet.h"
@@ -25,9 +25,13 @@
 #include "core/core.h"
 #include "core/command.h"
 
+#include "os/file.h"
+
 #include <shlwapi.h>
 #include <fstream>
 
+#include "Richedit.h"
+
 using pws_os::CUUID;
 
 #ifdef _DEBUG
@@ -71,10 +75,10 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
   m_bWordWrap = prefs->GetPref(PWSprefs::NotesWordWrap);
 
   m_password = m_password2 = M_realpassword();
-  m_notes = m_notesww = M_realnotes().Left(MAXTEXTCHARS);
+  m_notes = M_realnotes().Left(MAXTEXTCHARS);
 
   // Set up right-click Notes context menu additions
-  std::vector<st_context_menu> vmenu_items(3);
+  std::vector<st_context_menu> vmenu_items;
 
   st_context_menu st_cm;
   std::wstring cs_menu_string;
@@ -83,28 +87,37 @@ CAddEdit_Basic::CAddEdit_Basic(CWnd *pParent, st_AE_master_data *pAEMD)
   st_cm.menu_string = cs_menu_string;
   st_cm.message_number = PWS_MSG_EDIT_WORDWRAP;
   st_cm.flags = m_bWordWrap ? MF_CHECKED : MF_UNCHECKED;
-  vmenu_items[0] = st_cm;
+  vmenu_items.push_back(st_cm);
 
-  LoadAString(cs_menu_string, IDS_SHOW_NOTES);
+  st_cm.Empty();
+  LoadAString(cs_menu_string, IDS_NOTESZOOMIN);
   st_cm.menu_string = cs_menu_string;
-  st_cm.message_number = PWS_MSG_EDIT_SHOWNOTES;
-  st_cm.flags = m_isNotesHidden ? MF_CHECKED : MF_UNCHECKED;
-  vmenu_items[1] = st_cm;
+  st_cm.message_number = PWS_MSG_CALL_NOTESZOOMIN;
+  st_cm.flags = 0;
+  st_cm.lParam = 1;
+  vmenu_items.push_back(st_cm);
 
+  st_cm.Empty();
+  LoadAString(cs_menu_string, IDS_NOTESZOOMOUT);
+  st_cm.menu_string = cs_menu_string;
+  st_cm.message_number = PWS_MSG_CALL_NOTESZOOMOUT;
+  st_cm.flags = 0;
+  st_cm.lParam = -1;
+  vmenu_items.push_back(st_cm);
+
+  st_cm.Empty();
   LoadAString(cs_menu_string, IDS_EDITEXTERNALLY);
   st_cm.menu_string = cs_menu_string;
   st_cm.message_number = PWS_MSG_CALL_EXTERNAL_EDITOR;
   st_cm.flags = 0;
-  vmenu_items[2] = st_cm;
+  vmenu_items.push_back(st_cm);
 
-  m_pex_notes = new CEditExtn(vmenu_items);
-  m_pex_notesww = new CEditExtn(vmenu_items);
+  m_pex_notes = new CRichEditExtn(vmenu_items);
 }
 
 CAddEdit_Basic::~CAddEdit_Basic()
 {
   delete m_pex_notes;
-  delete m_pex_notesww;
   delete m_pToolTipCtrl;
 }
 
@@ -117,7 +130,6 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
   m_ex_password2.DoDDX(pDX, m_password2);
 
   DDX_Text(pDX, IDC_NOTES, (CString&)m_notes);
-  DDX_Text(pDX, IDC_NOTESWW, (CString&)m_notesww);
 
   DDX_CBString(pDX, IDC_GROUP, (CString&)M_group());
   DDX_Text(pDX, IDC_TITLE, (CString&)M_title());
@@ -131,7 +143,6 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
   DDX_Control(pDX, IDC_PASSWORD, m_ex_password);
   DDX_Control(pDX, IDC_PASSWORD2, m_ex_password2);
   DDX_Control(pDX, IDC_NOTES, *m_pex_notes);
-  DDX_Control(pDX, IDC_NOTESWW, *m_pex_notesww);
   DDX_Control(pDX, IDC_URL, m_ex_URL);
   DDX_Control(pDX, IDC_EMAIL, m_ex_email);
   DDX_Control(pDX, IDC_VIEWDEPENDENTS, m_ViewDependentsBtn);
@@ -157,7 +168,7 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
   ON_BN_CLICKED(ID_HELP, OnHelp)
 
   ON_BN_CLICKED(IDC_SHOWPASSWORD, OnShowPassword)
-  ON_BN_CLICKED(IDC_RANDOM, OnRandom)
+  ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
   ON_BN_CLICKED(IDC_LAUNCH, OnLaunch)
   ON_BN_CLICKED(IDC_SENDEMAIL, OnSendEmail)
   ON_BN_CLICKED(IDC_VIEWDEPENDENTS, OnViewDependents)
@@ -168,7 +179,6 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
   ON_EN_CHANGE(IDC_USERNAME, OnChanged)
   ON_EN_CHANGE(IDC_PASSWORD2, OnChanged)
   ON_EN_CHANGE(IDC_NOTES, OnENChangeNotes)
-  ON_EN_CHANGE(IDC_NOTESWW, OnENChangeNotes)
 
   ON_EN_CHANGE(IDC_URL, OnENChangeURL)
   ON_EN_CHANGE(IDC_EMAIL, OnENChangeEmail)
@@ -177,16 +187,15 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
   ON_EN_SETFOCUS(IDC_PASSWORD, OnENSetFocusPassword)
   ON_EN_SETFOCUS(IDC_PASSWORD2, OnENSetFocusPassword2)
   ON_EN_SETFOCUS(IDC_NOTES, OnENSetFocusNotes)
-  ON_EN_SETFOCUS(IDC_NOTESWW, OnENSetFocusNotes)
   ON_EN_KILLFOCUS(IDC_NOTES, OnENKillFocusNotes)
-  ON_EN_KILLFOCUS(IDC_NOTESWW, OnENKillFocusNotes)
 
   ON_CONTROL_RANGE(STN_CLICKED, IDC_STATIC_GROUP, IDC_STATIC_EMAIL, OnSTCExClicked)
 
   ON_MESSAGE(PWS_MSG_CALL_EXTERNAL_EDITOR, OnCallExternalEditor)
   ON_MESSAGE(PWS_MSG_EXTERNAL_EDITOR_ENDED, OnExternalEditorEnded)
   ON_MESSAGE(PWS_MSG_EDIT_WORDWRAP, OnWordWrap)
-  ON_MESSAGE(PWS_MSG_EDIT_SHOWNOTES, OnShowNotes)
+  ON_MESSAGE(PWS_MSG_CALL_NOTESZOOMIN, OnZoomNotes)
+  ON_MESSAGE(PWS_MSG_CALL_NOTESZOOMOUT, OnZoomNotes)
 
   // Common
   ON_MESSAGE(PSM_QUERYSIBLINGS, OnQuerySiblings)
@@ -202,8 +211,14 @@ BOOL CAddEdit_Basic::OnInitDialog()
 
   ModifyStyleEx(0, WS_EX_CONTROLPARENT);
 
-  ApplyPasswordFont(GetDlgItem(IDC_PASSWORD));
-  ApplyPasswordFont(GetDlgItem(IDC_PASSWORD2));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD2));
+
+  // Need to get change notifcations
+  m_pex_notes->SetEventMask(ENM_CHANGE | m_pex_notes->GetEventMask());
+
+  // Set plain text - not that it seems to do much!
+  m_pex_notes->SetTextMode(TM_PLAINTEXT);
 
   if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0) {
     GetDlgItem(IDC_STATIC_PROTECTED)->ShowWindow(SW_SHOW);
@@ -261,9 +276,9 @@ BOOL CAddEdit_Basic::OnInitDialog()
   if (M_uicaller() == IDS_VIEWENTRY ||
       (M_uicaller() == IDS_EDITENTRY && M_protected() != 0)) {
     // Change 'OK' to 'Close' and disable 'Cancel'
-    // "CancelToClose"  disables the System Command SC_CLOSE
+    // "CancelToClose" disables the System Command SC_CLOSE
     // from clicking X on PropertySheet so have implemented
-    // CAddEdit_PropertySheet::OnSysCommand to deal with it
+    // CAddEdit_PropertySheet::OnSysCommand to deal with it.
     CancelToClose();
 
     // Disable Group Combo
@@ -275,20 +290,13 @@ BOOL CAddEdit_Basic::OnInitDialog()
     GetDlgItem(IDC_PASSWORD)->SendMessage(EM_SETREADONLY, TRUE, 0);
     GetDlgItem(IDC_PASSWORD2)->SendMessage(EM_SETREADONLY, TRUE, 0);
     GetDlgItem(IDC_NOTES)->SendMessage(EM_SETREADONLY, TRUE, 0);
-    GetDlgItem(IDC_NOTESWW)->SendMessage(EM_SETREADONLY, TRUE, 0);
     GetDlgItem(IDC_URL)->SendMessage(EM_SETREADONLY, TRUE, 0);
     GetDlgItem(IDC_EMAIL)->SendMessage(EM_SETREADONLY, TRUE, 0);
 
     // Disable Button
-    GetDlgItem(IDC_RANDOM)->EnableWindow(FALSE);
+    GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
   }
 
-  m_pex_notes->EnableWindow(m_bWordWrap ? FALSE : TRUE);
-  m_pex_notes->ShowWindow(m_bWordWrap ? SW_HIDE : SW_SHOW);
-
-  m_pex_notesww->EnableWindow(m_bWordWrap ? TRUE : FALSE);
-  m_pex_notesww->ShowWindow(m_bWordWrap ? SW_SHOW : SW_HIDE);
-
   // Populate the combo box
   m_ex_group.ResetContent(); // groups might be from a previous DB (BR 3062758)
 
@@ -363,6 +371,26 @@ BOOL CAddEdit_Basic::OnInitDialog()
     HideNotes();
   }
 
+  CHARFORMAT cf = {0};
+  cf.cbSize = sizeof(cf);
+
+  m_pex_notes->SetSel(0, -1);
+  m_pex_notes->SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
+  m_iPointSize = cf.yHeight / 20;
+  m_pex_notes->Clear();
+
+  if (m_isNotesHidden) {
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
+  } else {
+    // If at the limit - don't allow to be called again in that direction
+    SetZoomMenu();
+  }
+
+  // Set initial Word Wrap
+  m_pex_notes->SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
+  m_pex_notes->UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
+
   UpdateData(FALSE);
   m_bInitdone = true;
   return TRUE;
@@ -375,7 +403,7 @@ void CAddEdit_Basic::OnHelp()
   HtmlHelp(DWORD_PTR((LPCWSTR)cs_HelpTopic), HH_DISPLAY_TOPIC);
 }
 
-HBRUSH CAddEdit_Basic::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
+HBRUSH CAddEdit_Basic::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
 {
   HBRUSH hbr = CAddEdit_PropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);
 
@@ -497,27 +525,32 @@ BOOL CAddEdit_Basic::PreTranslateMessage(MSG* pMsg)
   if (m_pToolTipCtrl != NULL)
     m_pToolTipCtrl->RelayEvent(pMsg);
 
-  // if user hit Ctrl+A in Notes control, then SelectAllNotes
-  if (pMsg->message == WM_KEYDOWN && pMsg->wParam == 'A' &&
-      (GetKeyState(VK_CONTROL) & 0x8000) &&
+  // Ctrl + 'key' in Notes
+  if (pMsg->message == WM_KEYDOWN &&
+      (GetKeyState(VK_CONTROL) & 0x8000)  == 0x8000 &&
       m_pex_notes->m_hWnd == ::GetFocus()) {
-    SelectAllNotes();
-    return TRUE;
+    switch (pMsg->wParam) {
+      case 'A':
+        // Ctrl+A (Select All), then SelectAllNotes
+        SelectAllNotes();
+        return TRUE;
+      case 'V':
+        // Ctrl+V (Paste), then do PasteSpecial
+        m_pex_notes->PasteSpecial(CF_UNICODETEXT);
+        return TRUE;
+      case VK_ADD:
+      case VK_SUBTRACT:
+        // Zoom in/out
+        OnZoomNotes(0, pMsg->wParam == VK_ADD ? 1 : -1);
+        return TRUE;
+    }
   }
-
+  
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_CONTROL &&
-      !m_bLaunchPlus && GetDlgItem(IDC_LAUNCH)->IsWindowEnabled()) {
-    CString cs_text(MAKEINTRESOURCE(IDS_LAUNCHPLUS));
+      GetDlgItem(IDC_LAUNCH)->IsWindowEnabled()) {
+    CString cs_text(MAKEINTRESOURCE(m_bLaunchPlus ? IDS_LAUNCH : IDS_LAUNCHPLUS));
     GetDlgItem(IDC_LAUNCH)->SetWindowText(cs_text);
-    m_bLaunchPlus = true;
-    return TRUE;
-  }
-
-  if (pMsg->message == WM_KEYUP && pMsg->wParam == VK_CONTROL &&
-      m_bLaunchPlus && GetDlgItem(IDC_LAUNCH)->IsWindowEnabled()) {
-    CString cs_text(MAKEINTRESOURCE(IDS_LAUNCH));
-    GetDlgItem(IDC_LAUNCH)->SetWindowText(cs_text);
-    m_bLaunchPlus = false;
+    m_bLaunchPlus = !m_bLaunchPlus;
     return TRUE;
   }
 
@@ -544,7 +577,6 @@ BOOL CAddEdit_Basic::OnApply()
   M_symbols().EmptyIfOnlyWhiteSpace();
 
   m_notes.EmptyIfOnlyWhiteSpace();
-  m_notesww.EmptyIfOnlyWhiteSpace();
 
   if (m_password.IsOnlyWhiteSpace()) {
     m_password.Empty();
@@ -556,7 +588,7 @@ BOOL CAddEdit_Basic::OnApply()
     M_realpassword() = m_password;
 
   if (!m_isNotesHidden)
-    M_realnotes() = m_bWordWrap ? m_notesww : m_notes;
+    M_realnotes() = m_notes;
 
   UpdateData(FALSE);
 
@@ -719,18 +751,40 @@ void CAddEdit_Basic::HidePassword()
   m_ex_password2.Invalidate();
 }
 
-LRESULT CAddEdit_Basic::OnShowNotes(WPARAM, LPARAM)
+void CAddEdit_Basic::SetZoomMenu()
 {
-  UpdateData(TRUE);
-  if (m_isNotesHidden) {
-    ShowNotes();
-  } else {
-    HideNotes();
-  }
-  UpdateData(FALSE);
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, m_iPointSize < 72);
+    m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, m_iPointSize > 6);
+}
 
-  m_pex_notes->UpdateState(PWS_MSG_EDIT_SHOWNOTES, m_isNotesHidden ? FALSE : TRUE);
-  m_pex_notesww->UpdateState(PWS_MSG_EDIT_SHOWNOTES, m_isNotesHidden ? FALSE : TRUE);
+LRESULT CAddEdit_Basic::OnZoomNotes(WPARAM, LPARAM lParam)
+{
+  // Zoom into and out of notes (by menu and also by keyboard)
+  UpdateData(TRUE);
+
+  ASSERT(lParam != 0);
+
+  if ((lParam < 0 && m_iPointSize <= 6) || (lParam > 0 && m_iPointSize >= 72))
+    return 0L;
+
+  WPARAM wp_increment = (lParam > 0 ? 1 : -1) * 2;
+
+  long nStart, nEnd;
+  m_pex_notes->GetSel(nStart, nEnd);
+
+  m_pex_notes->SetSel(0, -1);
+  m_pex_notes->HideSelection(TRUE, FALSE);
+  m_pex_notes->SendMessage(EM_SETFONTSIZE, wp_increment, 0);
+
+  CHARFORMAT cf = {0};
+  cf.cbSize = sizeof(cf);
+
+  m_pex_notes->SendMessage(EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
+  m_iPointSize = cf.yHeight / 20;
+
+  m_pex_notes->SetSel(nStart, nEnd);
+
+  SetZoomMenu();
   return 0L;
 }
 
@@ -738,33 +792,28 @@ void CAddEdit_Basic::ShowNotes()
 {
   m_isNotesHidden = false;
   m_notes = M_realnotes();
-  m_notesww = M_realnotes();
 
   ((CEdit *)GetDlgItem(IDC_NOTES))->Invalidate();
-  ((CEdit *)GetDlgItem(IDC_NOTESWW))->Invalidate();
- }
+
+  // If at the limit - don't allow to be called again in that direction
+  SetZoomMenu();
+}
 
 void CAddEdit_Basic::HideNotes()
 {
   m_isNotesHidden = true;
-  if (m_bWordWrap) {
-    if (m_notesww != HIDDEN_NOTES) {
-      M_realnotes() = m_notesww;
-    }
-  } else {
-    if (m_notes != HIDDEN_NOTES) {
-      M_realnotes() = m_notes;
-    }
-  }
+  M_realnotes() = m_notes;
 
   m_notes = HIDDEN_NOTES;
-  m_notesww = HIDDEN_NOTES;
 
   ((CEdit *)GetDlgItem(IDC_NOTES))->Invalidate();
-  ((CEdit *)GetDlgItem(IDC_NOTESWW))->Invalidate();
+
+  // Disable zoom of hidden text
+  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMIN, false);
+  m_pex_notes->EnableMenuItem(PWS_MSG_CALL_NOTESZOOMOUT, false);
 }
 
-void CAddEdit_Basic::OnRandom()
+void CAddEdit_Basic::OnGeneratePassword()
 {
   UpdateData(TRUE);
 
@@ -796,7 +845,17 @@ void CAddEdit_Basic::OnRandom()
   }
 
   StringX passwd;
-  M_pDbx()->MakeRandomPassword(passwd, M_pwp(), M_symbols());
+  if (M_ipolicy() == NAMED_POLICY) {
+    PWPolicy st_pp;
+    M_pDbx()->GetPolicyFromName(M_policyname(), st_pp);
+    M_pDbx()->MakeRandomPassword(passwd, st_pp);
+  } else {
+    // XXX temp - to be cleaned up
+    PWPolicy policy(M_pwp());
+    policy.symbols = LPCWSTR(M_symbols());
+    M_pDbx()->MakeRandomPassword(passwd, policy);
+  }
+
   if (rc == CChangeAliasPswd::CHANGEBASE) {
     // Change Base
     ItemListIter iter = M_pDbx()->Find(M_base_uuid());
@@ -837,6 +896,12 @@ void CAddEdit_Basic::OnENChangeNotes()
   if (!m_bInitdone || m_AEMD.uicaller != IDS_EDITENTRY)
     return;
 
+  // Called for any change - even just clicking on it - so check really changed
+  CSecString current_notes;
+  m_pex_notes->GetWindowText(current_notes);
+  if (current_notes == M_realnotes())
+    return;
+
   m_ae_psh->SetChanged(true);
   m_ae_psh->SetNotesChanged(true); // Needed if Notes field is long and will be truncated
   UpdateData(TRUE);
@@ -860,45 +925,45 @@ void CAddEdit_Basic::OnSTCExClicked(UINT nID)
 {
   UpdateData(TRUE);
 
-  StringX cs_data;
+  CSecString cs_data;
   int iaction(0);
 
   // NOTE: These values must be contiguous in "resource.h"
   switch (nID) {
     case IDC_STATIC_GROUP:
       m_stc_group.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = StringX(M_group());
+      cs_data = M_group();
       iaction = CItemData::GROUP;
       break;
     case IDC_STATIC_TITLE:
       m_stc_title.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = StringX(M_title());
+      cs_data = M_title();
       iaction = CItemData::TITLE;
       break;
     case IDC_STATIC_USERNAME:
       m_stc_username.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = StringX(M_username());
+      cs_data = M_username();
       iaction = CItemData::USER;
       break;
     case IDC_STATIC_PASSWORD:
       m_stc_password.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = StringX(M_realpassword());
+      cs_data = M_realpassword();
       iaction = CItemData::PASSWORD;
       break;
     case IDC_STATIC_NOTES:
       m_stc_notes.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
-      cs_data = StringX(M_realnotes());
+      cs_data = M_realnotes();
       iaction = CItemData::NOTES;
       break;
     case IDC_STATIC_URL:
-      cs_data = StringX(M_URL());
+      cs_data = M_URL();
       m_stc_URL.FlashBkgnd(CAddEdit_PropertyPage::crefGreen);
       iaction = CItemData::URL;
       break;
     case IDC_STATIC_EMAIL:
-      cs_data = StringX(M_email());
+      cs_data = M_email();
       // If Ctrl pressed - also copy to URL field with the 'mailto:' prefix
-      if (GetKeyState(VK_CONTROL) != 0 && !M_email().IsEmpty()) {
+      if ((GetKeyState(VK_CONTROL) & 0x8000) != 0 && !M_email().IsEmpty()) {
         M_URL() = L"mailto:" + cs_data;
         UpdateData(FALSE);
       }
@@ -916,33 +981,16 @@ void CAddEdit_Basic::SelectAllNotes()
 {
   // Here from PreTranslateMessage iff User pressed Ctrl+A
   // in Notes control
-  ((CEdit *)GetDlgItem(m_bWordWrap ? IDC_NOTESWW : IDC_NOTES))->
-                       SetSel(0, -1, TRUE);
+  ((CEdit *)GetDlgItem(IDC_NOTES))->SetSel(0, -1, TRUE);
 }
 
 LRESULT CAddEdit_Basic::OnWordWrap(WPARAM, LPARAM)
 {
-  // Get value of notes from dialog.
-  UpdateData(TRUE);
-  if (m_bWordWrap)
-    m_notes = m_notesww;
-  else
-    m_notesww = m_notes;
-
   m_bWordWrap = !m_bWordWrap;
-  // Update dalog
-  UpdateData(FALSE);
+  m_pex_notes->SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
 
-  m_pex_notes->EnableWindow(m_bWordWrap ? FALSE : TRUE);
-  m_pex_notes->ShowWindow(m_bWordWrap ? SW_HIDE : SW_SHOW);
-  m_pex_notes->Invalidate();
   m_pex_notes->UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
 
-  m_pex_notesww->EnableWindow(m_bWordWrap ? TRUE : FALSE);
-  m_pex_notesww->ShowWindow(m_bWordWrap ? SW_SHOW : SW_HIDE);
-  m_pex_notesww->Invalidate();
-  m_pex_notesww->UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
-
   return 0L;
 }
 
@@ -958,10 +1006,10 @@ void CAddEdit_Basic::OnENSetFocusNotes()
 void CAddEdit_Basic::OnENKillFocusNotes()
 {
   UpdateData(TRUE);
-  if (m_isNotesHidden) {
+  if (!m_isNotesHidden) {
     HideNotes();
   } else
-    M_realnotes() = m_bWordWrap ? m_notesww : m_notes;
+    M_realnotes() = m_notes;
 
   UpdateData(FALSE);
 }
@@ -969,7 +1017,6 @@ void CAddEdit_Basic::OnENKillFocusNotes()
 void CAddEdit_Basic::OnLaunch()
 {
   UpdateData(TRUE);
-  StringX sx_url = StringX(M_URL());
   std::vector<size_t> vactionverboffsets;
   StringX sx_autotype = PWSAuxParse::GetAutoTypeString(M_autotype(),
                                                        M_group(),
@@ -981,7 +1028,7 @@ void CAddEdit_Basic::OnLaunch()
 
   const bool bDoAutoType = (GetKeyState(VK_CONTROL) & 0x8000) != 0;
 
-  M_pDbx()->LaunchBrowser(sx_url.c_str(), sx_autotype, vactionverboffsets, bDoAutoType);
+  M_pDbx()->LaunchBrowser(M_URL(), sx_autotype, vactionverboffsets, bDoAutoType);
   M_pDbx()->UpdateLastClipboardAction(CItemData::URL);
 
   if (bDoAutoType) {
@@ -999,9 +1046,8 @@ void CAddEdit_Basic::OnLaunch()
 void CAddEdit_Basic::OnSendEmail()
 {
   UpdateData(TRUE);
-  StringX sx_email = StringX(M_email());
 
-  M_pDbx()->SendEmail(sx_email.c_str());
+  M_pDbx()->SendEmail(M_email());
   M_pDbx()->UpdateLastClipboardAction(CItemData::EMAIL);
 }
 
@@ -1045,14 +1091,22 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
                   0,                     // create unique name
                   self->m_szTempName);   // buffer for name
 
-  // Open it and put the Notes field in it
-  std::wofstream ofs(self->m_szTempName);
-  if (ofs.bad())
-    return 16;
+  FILE *fd;
 
-  ofs << LPCWSTR(self->M_realnotes()) << std::endl;
-  ofs.flush();
-  ofs.close();
+  if ((fd = pws_os::FOpen(self->m_szTempName, _T("w+b"))) == NULL) {
+    return false;
+  }
+
+  // Write BOM
+  const unsigned int iBOM = 0xFEFF;
+  putwc(iBOM, fd);
+
+  // Write out text
+  fwrite(reinterpret_cast<const void *>((LPCWSTR)self->M_realnotes()), sizeof(BYTE),
+             self->M_realnotes().GetLength() * sizeof(TCHAR), fd);
+
+  // Close file before invoking editor
+  fclose(fd);
 
   StringX sxEditor = PWSprefs::GetInstance()->GetPref(PWSprefs::AltNotesEditor);
   if (sxEditor.empty()) {
@@ -1116,24 +1170,45 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM, LPARAM)
   GetDlgItem(IDC_NOTES)->EnableWindow(TRUE);
 
   // Now get what the user saved in this file and put it back into Notes field
-  std::wifstream ifs(m_szTempName);
-  if (ifs.bad())
-    return 16;
-
   M_realnotes().Empty();
   std::wstring linebuf, note;
 
-  // Get first line
-  getline(ifs, note, L'\n');
+  FILE *fd;
 
-  // Now get the rest (if any)
-  while (!ifs.eof()) {
-    getline(ifs, linebuf, L'\n');
-    note += L"\r\n";
-    note += linebuf;
+  if ((fd = pws_os::FOpen(m_szTempName, _T("r+b"))) == NULL) {
+    return 0L;
   }
 
-  ifs.close();
+  long flength = pws_os::fileLength(fd);
+
+  ASSERT(flength % 2 == 0);
+
+  BYTE *pBuffer = new BYTE[flength + sizeof(wchar_t)];
+  memset(pBuffer, 0, flength + sizeof(wchar_t));
+
+  if (flength >= 2) {
+    // Read in BOM and check it is
+    const unsigned char BOM[] = {0xff, 0xfe};
+    fread(pBuffer, 1, 2, fd);
+
+    // if not a BOM - backspace and treat as data
+    if (pBuffer[0] != BOM[0] || pBuffer[1] != BOM[1]) {
+      fseek(fd, 0, SEEK_SET);
+      flength += 2;
+    }
+  }
+
+  // Clear BOM
+  memset(pBuffer, 0, 2);
+  // Read in text
+  fread(pBuffer, sizeof(BYTE), flength - 2, fd);
+
+  note = reinterpret_cast<const LPCWSTR>(pBuffer);
+
+  delete [] pBuffer;
+
+  // Close file before invoking editor
+  fclose(fd);
 
   if (note.length() > MAXTEXTCHARS) {
     note = note.substr(0, MAXTEXTCHARS);
@@ -1146,11 +1221,10 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM, LPARAM)
 
   // Set real notes field, and
   // we are still displaying the old text, so replace that too
-  M_realnotes() = m_notes = m_notesww = note.c_str();
+  M_realnotes() = m_notes = note.c_str();
 
   UpdateData(FALSE);
   ((CEdit*)GetDlgItem(IDC_NOTES))->Invalidate();
-  ((CEdit*)GetDlgItem(IDC_NOTESWW))->Invalidate();
 
   // Delete temporary file
   _wremove(m_szTempName);
