@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -114,7 +114,7 @@ LRESULT CCoolMenuManager::WindowProc(UINT msg, WPARAM wp, LPARAM lp)
 //////////////////
 // Get menu font, creating if needed
 //
-CFont* CCoolMenuManager::GetMenuFont()
+CFont * CCoolMenuManager::GetMenuFont()
 {
   if (!(HFONT)m_fontMenu) {
     NONCLIENTMETRICS ncm;
@@ -131,9 +131,8 @@ CFont* CCoolMenuManager::GetMenuFont()
 BOOL CCoolMenuManager::CMOnMeasureItem(LPMEASUREITEMSTRUCT lpmis)
 {
   ASSERT(lpmis);
-  CMenuItemData* pmd = (CMenuItemData*)lpmis->itemData;
-  ASSERT(pmd);
-  if (lpmis->CtlType != ODT_MENU || !pmd->IsCMID())
+  CMenuItemData *pmd = (CMenuItemData *)lpmis->itemData;
+  if (lpmis->CtlType != ODT_MENU || pmd == NULL || !pmd->IsCMID())
     return FALSE; // not handled by me
 
   if (pmd->fType & MFT_SEPARATOR) {
@@ -145,7 +144,7 @@ BOOL CCoolMenuManager::CMOnMeasureItem(LPMEASUREITEMSTRUCT lpmis)
 
     CWindowDC dc(NULL);  // screen DC--I won't actually draw on it
     CRect rcText(0, 0, 0, 0);
-    CFont* pOldFont = dc.SelectObject(GetMenuFont());
+    CFont *pOldFont = dc.SelectObject(GetMenuFont());
     dc.DrawText(pmd->text, rcText, DT_MYSTANDARD|DT_CALCRECT);
     dc.SelectObject(pOldFont);
 
@@ -174,9 +173,8 @@ BOOL CCoolMenuManager::CMOnMeasureItem(LPMEASUREITEMSTRUCT lpmis)
 BOOL CCoolMenuManager::CMOnDrawItem(LPDRAWITEMSTRUCT lpdis)
 {
   ASSERT(lpdis);
-  CMenuItemData* pmd = (CMenuItemData*)lpdis->itemData;
-  ASSERT(pmd);
-  if (lpdis->CtlType != ODT_MENU || !pmd->IsCMID())
+  CMenuItemData *pmd = (CMenuItemData *)lpdis->itemData;
+  if (lpdis->CtlType != ODT_MENU || pmd == NULL || !pmd->IsCMID())
     return FALSE; // not handled by me
 
   ASSERT(lpdis->itemAction != ODA_FOCUS);
