@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -35,8 +35,6 @@
 #include "DDStatic.h"
 #include "MenuShortcuts.h"
 #include "AdvancedDlg.h"
-#include "AddEdit_PropertySheet.h"
-#include "CompareResultsDlg.h"
 
 #include "core/UIinterface.h"
 #include "core/PWScore.h"
@@ -47,6 +45,7 @@
 #include "core/Command.h"
 
 #include "os/run.h"
+#include "os/UUID.h"
 
 #include <vector>
 #include <map>
@@ -71,25 +70,24 @@
 
 #endif  /* WINVER < 0x0501 */
 
-class CDDObList;
-
 #if defined(POCKET_PC) || (_MFC_VER <= 1200)
 DECLARE_HANDLE(HDROP);
 #endif
 
-// custom message event used for system tray handling.
+// Custom message event used for system tray handling
 #define PWS_MSG_ICON_NOTIFY             (WM_APP + 10)
 
-// to catch post Header drag
+// To catch post Header drag
 #define PWS_MSG_HDR_DRAG_COMPLETE       (WM_APP + 20)
 #define PWS_MSG_CCTOHDR_DD_COMPLETE     (WM_APP + 21)
 #define PWS_MSG_HDRTOCC_DD_COMPLETE     (WM_APP + 22)
 
 // Process Compare Result Dialog click/menu functions
 #define PWS_MSG_COMPARE_RESULT_FUNCTION (WM_APP + 30)
+#define PWS_MSG_COMPARE_RESULT_ALLFNCTN (WM_APP + 31)
 
 // Equivalent one from Expired Password dialog
-#define PWS_MSG_EXPIRED_PASSWORD_EDIT   (WM_APP + 31)
+#define PWS_MSG_EXPIRED_PASSWORD_EDIT   (WM_APP + 32)
 
 // Edit/Add extra context menu messages
 #define PWS_MSG_CALL_EXTERNAL_EDITOR    (WM_APP + 40)
@@ -97,10 +95,15 @@ DECLARE_HANDLE(HDROP);
 #define PWS_MSG_EDIT_WORDWRAP           (WM_APP + 42)
 #define PWS_MSG_EDIT_SHOWNOTES          (WM_APP + 43)
 #define PWS_MSG_EDIT_APPLY              (WM_APP + 44)
+#define PWS_MSG_CALL_NOTESZOOMIN        (WM_APP + 45)
+#define PWS_MSG_CALL_NOTESZOOMOUT       (WM_APP + 46)
 
 // Simulate Ctrl+F from Find Toolbar "enter"
 #define PWS_MSG_TOOLBAR_FIND            (WM_APP + 50)
 
+// Perform Drag Autotype
+#define PWS_MSG_DRAGAUTOTYPE            (WM_APP + 55)
+
 // Update current filters whilst SetFilters dialog is open
 #define PWS_MSG_EXECUTE_FILTERS         (WM_APP + 60)
 
@@ -108,29 +111,37 @@ DECLARE_HANDLE(HDROP);
 #define PWS_MSG_INSERTBUFFER            (WM_APP + 70)
 */
 
-/* timer event number used to by PupText.  Here for doc. only
+
+/*
+  Timer related values (note - all documented her but some defined only where needed.
+*/
+
+/* Timer event number used to by PupText.  Here for doc. only
 #define TIMER_PUPTEXT             0x03 */
-// timer event number used to check if the workstation is locked
+// Timer event number used to check if the workstation is locked
 #define TIMER_LOCKONWTSLOCK       0x04
-// timer event number used to support lock on user-defined idle timeout
+// Timer event number used to support lock on user-defined idle timeout
 #define TIMER_LOCKDBONIDLETIMEOUT 0x05
 // Definition of a minute in milliseconds
 #define MINUTE 60000
 // How ofter should idle timeout timer check:
 #define IDLE_CHECK_RATE 2
 #define IDLE_CHECK_INTERVAL (MINUTE/IDLE_CHECK_RATE)
-// timer event number used to support Find in PWListCtrl when icons visible
+// Timer event number used to support Find in PWListCtrl when icons visible
 #define TIMER_FIND                0x06
-// timer event number used to support display of notes in List & Tree controls
+// Timer event number used to support display of notes in List & Tree controls
 #define TIMER_ND_HOVER            0x07
 #define TIMER_ND_SHOWING          0x08
-// timer event number used to support DragBar
+// Timer event number used to support DragBar
 #define TIMER_DRAGBAR             0x09
-/* timer event numbers used to by ControlExtns for ListBox tooltips.  Here for doc. only
+/* Timer event numbers used to by ControlExtns for ListBox tooltips.  Here for doc. only
 #define TIMER_LB_HOVER            0x0A
-#define TIMER_LB_SHOWING          0x0B */
+#define TIMER_LB_SHOWING          0x0B 
+/* Timer event numbers used by StatusBar for tooltips.  Here for doc. only
+#define TIMER_SB_HOVER            0x0C
+#define TIMER_SB_SHOWING          0x0D */
 // Timer event for daily expired entries check
-#define TIMER_EXPENT              0x0C
+#define TIMER_EXPENT              0x0E
 
 /*
 HOVER_TIME_ND       The length of time the pointer must remain stationary
@@ -177,14 +188,18 @@ enum PopupMenus {FILEMENU = 0, EXPORTMENU, IMPORTMENU,
 enum {GCP_FIRST      = 0,   // At startup of PWS
       GCP_NORMAL     = 1,   // Only OK, CANCEL & HELP buttons
       GCP_RESTORE    = 2,   // Only OK, CANCEL & HELP buttons
-      GCP_WITHEXIT   = 3};  // OK, CANCEL, EXIT & HELP buttons
+      GCP_WITHEXIT   = 3,   // OK, CANCEL, EXIT & HELP buttons
+      GCP_CHANGEMODE = 4};  // Only OK, CANCEL & HELP buttons
 
 // GCP read only flags - tested via AND, set via OR, must be power of 2.
 enum {GCP_READONLY = 1,
       GCP_FORCEREADONLY = 2,
       GCP_HIDEREADONLY = 4};
 
+class CDDObList;
 class ExpiredList;
+class CAddEdit_PropertySheet;
+
 //-----------------------------------------------------------------------------
 class DboxMain : public CDialog, public UIInterFace
 {
@@ -212,7 +227,7 @@ public:
   ItemListIter Find(int i);
 
   // Find entry by UUID
-  ItemListIter Find(const uuid_array_t &uuid)
+  ItemListIter Find(const pws_os::CUUID &uuid)
   {return m_core.Find(uuid);}
 
   // End of list markers
@@ -236,6 +251,9 @@ public:
   // Count the number of total entries.
   size_t GetNumEntries() const {return m_core.GetNumEntries();}
 
+  // Has the GUI been built and has entries
+  bool IsGUIEmpty() const { return m_ctlItemTree.GetCount() == 0;}
+
   // Get CItemData @ position
   CItemData &GetEntryAt(ItemListIter iter) const
   {return m_core.GetEntry(iter);}
@@ -251,8 +269,7 @@ public:
   BOOL SelectFindEntry(const int i, BOOL MakeVisible = FALSE);
   void SelectFirstEntry();
 
-  int CheckPasskey(const StringX &filename, const StringX &passkey)
-  {return m_core.CheckPasskey(filename, passkey);}
+  int CheckPasskey(const StringX &filename, const StringX &passkey, PWScore *pcore = NULL);
   enum ChangeType {Clear, Data, TimeStamp, DBPrefs, ClearDBPrefs};
   void SetChanged(ChangeType changed);
   void ChangeOkUpdate();
@@ -282,12 +299,10 @@ public:
   void InvalidateSearch() {m_FindToolBar.InvalidateSearch();}
   void ResumeOnDBNotification() {m_core.ResumeOnDBNotification();}
   void SuspendOnDBNotification() {m_core.SuspendOnDBNotification();}
-  bool IsDBReadOnly() const {return m_core.IsReadOnly();};
+  bool IsDBReadOnly() const {return m_core.IsReadOnly();}
   void SetStartSilent(bool state);
   void SetStartClosed(bool state) {m_IsStartClosed = state;}
-  void SetValidate(bool state) {m_bValidate = state;}
-  void MakeRandomPassword(StringX& password, PWPolicy &pwp, stringT st_symbols,
-                          bool bIssueMsg = false);
+  void MakeRandomPassword(StringX &password, PWPolicy &pwp, bool bIssueMsg = false);
   BOOL LaunchBrowser(const CString &csURL, const StringX &sxAutotype,
                      const std::vector<size_t> &vactionverboffsets,
                      const bool &bDoAutotype);
@@ -301,7 +316,7 @@ public:
   bool ClearClipboardData() {return m_clipboard.ClearData();}
   bool SetClipboardData(const StringX &data)
   {return m_clipboard.SetData(data.c_str());}
-  void AddEntries(CDDObList &in_oblist, const StringX &DropGroup);
+  void AddDDEntries(CDDObList &in_oblist, const StringX &DropGroup);
   StringX GetUniqueTitle(const StringX &group, const StringX &title,
                          const StringX &user, const int IDS_MESSAGE) const
   {return m_core.GetUniqueTitle(group, title, user, IDS_MESSAGE);}
@@ -329,7 +344,7 @@ public:
   void CreateShortcutEntry(CItemData *pci, const StringX &cs_group,
                            const StringX &cs_title, const StringX &cs_user,
                            StringX &sxNewDBPrefsString);
-  bool SetNotesWindow(const CPoint point, const bool bVisible = true);
+  bool SetNotesWindow(const CPoint ptClient, const bool bVisible = true);
   bool IsFilterActive() const {return m_bFilterActive;}
   int GetNumPassedFiltering() const {return m_bNumPassedFiltering;}
   CItemData *GetLastSelected() const;
@@ -344,15 +359,33 @@ public:
   void ClearFilter();
   void ExportFilters(PWSFilters &MapFilters);
 
-  void DoAutoType(const StringX &sx_autotype, const std::vector<size_t> &vactionverboffsets);
+  void DoAutoType(const StringX &sx_autotype, 
+                  const std::vector<size_t> &vactionverboffsets);
   void UpdateLastClipboardAction(const int iaction);
   void PlaceWindow(CWnd *pWnd, CRect *pRect, UINT uiShowCmd);
-  void SetDCAText(CItemData * pci = NULL);
-  void OnItemSelected(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  void SetDCAText(CItemData *pci = NULL);
+  void OnItemSelected(NMHDR *pNotifyStruct, LRESULT *pLResult, const bool bTreeView);
   bool IsNodeModified(StringX &path) const
   {return m_core.IsNodeModified(path);}
   StringX GetCurFile() const {return m_core.GetCurFile();}
 
+  bool EditItem(CItemData *pci, PWScore *pcore = NULL);
+  bool GetPolicyFromName(const StringX &sxPolicyName, PWPolicy &st_pp)
+  {return m_core.GetPolicyFromName(sxPolicyName, st_pp);}
+  void GetPolicyNames(std::vector<std::wstring> &vNames)
+  {m_core.GetPolicyNames(vNames);}
+  const PSWDPolicyMap &GetPasswordPolicies()
+  {return m_core.GetPasswordPolicies();}
+
+  bool IsEmptyGroup(const StringX &sxEmptyGroup)
+  {return m_core.IsEmptyGroup(sxEmptyGroup);}
+  
+  // Need this to be public
+  bool LongPPs(CWnd *pWnd);
+
+  bool GetShortCut(const unsigned int &uiMenuItem, unsigned short int &siVirtKey,
+                   unsigned char &cModifier);
+
   // Following to simplify Command creation in child dialogs:
   CommandInterface *GetCore() {return &m_core;}
   
@@ -363,18 +396,20 @@ public:
   void ViewReport(CReport &rpt) const;
   void SetUpdateWizardWindow(CWnd *pWnd)
   {m_pWZWnd = pWnd;}
+
   stringT DoMerge(PWScore *pothercore,
-                  const bool bAdvanced, CReport *prpt);
+                  const bool bAdvanced, CReport *prpt, bool *pbCancel);
   bool DoCompare(PWScore *pothercore,
-                 const bool bAdvanced, CReport *prpt);
+                 const bool bAdvanced, CReport *prpt, bool *pbCancel);
   void DoSynchronize(PWScore *pothercore,
-                     const bool bAdvanced, int &numUpdated, CReport *prpt);
+                     const bool bAdvanced, int &numUpdated, CReport *prpt, bool *pbCancel);
   int DoExportText(const StringX &sx_Filename, const bool bAll,
                    const wchar_t &delimiter, const bool bAdvanced,
                    int &numExported, CReport *prpt);
   int DoExportXML(const StringX &sx_Filename, const bool bAll,
                   const wchar_t &delimiter, const bool bAdvanced,
                   int &numExported, CReport *prpt);
+
   int TestSelection(const bool bAdvanced,
                     const stringT &subgroup_name,
                     const int &subgroup_object,
@@ -382,7 +417,10 @@ public:
                     const OrderedItemList *il) const
   {return m_core.TestSelection(bAdvanced, subgroup_name,
                                subgroup_object, subgroup_function, il);}
+
   void MakeOrderedItemList(OrderedItemList &il) const;
+  bool MakeMatchingGTUSet(GTUSet &setGTU, const StringX &sxPolicyName) const
+  {return m_core.InitialiseGTU(setGTU, sxPolicyName);}
   CItemData *getSelectedItem();
   void UpdateGUIDisplay();
   CString ShowCompareResults(const StringX sx_Filename1, const StringX sx_Filename2,
@@ -405,7 +443,8 @@ public:
   CLVHdrCtrl m_LVHdrCtrl;
   CColumnChooserDlg *m_pCC;
   CPoint m_RCMousePos;
-  CDDStatic m_DDGroup, m_DDTitle, m_DDUser, m_DDPassword, m_DDNotes, m_DDURL, m_DDemail;
+  CDDStatic m_DDGroup, m_DDTitle, m_DDUser, m_DDPassword, m_DDNotes, m_DDURL, m_DDemail,
+            m_DDAutotype;
   //}}AFX_DATA
 
   CRUEList m_RUEList;   // recent entry lists
@@ -413,10 +452,10 @@ public:
   wchar_t *m_eye_catcher;
 
   bool m_bDoAutoType;
-  StringX m_AutoType;
+  StringX m_sxAutoType;
   std::vector<size_t> m_vactionverboffsets;
 
-  // Used in Add & Edit & OptionsPasswordPolicy
+  // Used in Add & Edit & PasswordPolicyDlg
   PWPolicy m_pwp;
 
   // Mapping Group to Tree Item to save searching all the time!
@@ -428,11 +467,17 @@ public:
   bool CheckPreTranslateRename(MSG* pMsg);
   bool CheckPreTranslateAutoType(MSG* pMsg);
 
-  void SetSetup() {m_bSetup = true;} // called by app when '--setup' passed
+  void SetSetup() {m_bSetup = true;}                     // called by app when '--setup' passed
+  void NoValidation() {m_bNoValidation = true;}          // called by app when '--novalidate' passed
+  void AllowCompareEntries() {m_bCompareEntries = true;} // called by app when '--cetreeview' passed
 
   // Needed public function for ComapreResultsDialog
   void CPRInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
   {OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);}
+
+  const MapMenuShortcuts &GetMapMenuShortcuts() {return m_MapMenuShortcuts;}
+  const std::vector<UINT> &GetExcludedMenuItems() {return m_ExcludedMenuItems;}
+  const std::vector<st_MenuShortcut> &GetReservedShortcuts() {return m_ReservedShortcuts;}
   
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(DboxMain)
@@ -447,10 +492,11 @@ protected:
   bool m_bSizing;
   bool m_bIsRestoring;
   bool m_bOpen;
-  bool m_bValidate; // do validation after reading db
   bool m_bInRestoreWindowsData;
 
-  bool m_bSetup; // invoked with '--setup'?
+  bool m_bSetup;          // invoked with '--setup'?
+  bool m_bNoValidation;   // invoked with '--novalidate'?
+  bool m_bCompareEntries; // invoked with '--cetreeview'?
   
 #if !defined(POCKET_PC)
   CString m_titlebar; // what's displayed in the title bar
@@ -488,8 +534,7 @@ protected:
   bool m_bBoldItem;
 
   WCHAR *m_pwchTip;
-  char *m_pchTip;
-
+  
   StringX m_TreeViewGroup; // used by OnAdd & OnAddGroup
   CCoolMenuManager m_menuManager;
   CMenuTipManager m_menuTipManager;
@@ -524,14 +569,19 @@ protected:
   LRESULT OnTrayNotification(WPARAM wParam, LPARAM lParam);
 
   LRESULT OnProcessCompareResultFunction(WPARAM wParam, LPARAM lParam);
+  LRESULT OnProcessCompareResultAllFunction(WPARAM wParam, LPARAM lParam);
   LRESULT OnEditExpiredPasswordEntry(WPARAM wParam, LPARAM lParam);
-  LRESULT ViewCompareResult(PWScore *pcore, uuid_array_t &uuid);
-  LRESULT EditCompareResult(PWScore *pcore, uuid_array_t &uuid);
+  LRESULT ViewCompareResult(PWScore *pcore, const pws_os::CUUID &uuid);
+  LRESULT EditCompareResult(PWScore *pcore, const pws_os::CUUID &uuid);
   LRESULT CopyCompareResult(PWScore *pfromcore, PWScore *ptocore,
-                            uuid_array_t &fromuuid, uuid_array_t &touuid);
+                            const pws_os::CUUID &fromuuid, const pws_os::CUUID &touuid);
   LRESULT SynchCompareResult(PWScore *pfromcore, PWScore *ptocore,
-                             uuid_array_t &fromuuid, uuid_array_t &touuid);
+                             const pws_os::CUUID &fromuuid, const pws_os::CUUID &touuid);
+  LRESULT CopyAllCompareResult(WPARAM wParam);
+  LRESULT SynchAllCompareResult(WPARAM wParam);
+  
   LRESULT OnToolBarFindMessage(WPARAM wParam, LPARAM lParam);
+  LRESULT OnDragAutoType(WPARAM wParam, LPARAM lParam);
   LRESULT OnExecuteFilters(WPARAM wParam, LPARAM lParam);
   LRESULT OnApplyEditChanges(WPARAM wParam, LPARAM lParam);
 
@@ -564,16 +614,13 @@ protected:
   int New(void);
 
   void AutoType(const CItemData &ci);
-  bool EditItem(CItemData *pci, PWScore *pcore = NULL);
   void UpdateEntry(CAddEdit_PropertySheet *pentry_psh);
   bool EditShortcut(CItemData *pci, PWScore *pcore = NULL);
   void SetFindToolBar(bool bShow);
   void ApplyFilters();
 
-  HRGN GetWorkAreaRegion();
   void GetMonitorRect(HWND hwnd, RECT *prc, BOOL fWork);
   void ClipRectToMonitor(HWND hwnd, RECT *prc, BOOL fWork);
-  static BOOL CALLBACK EnumScreens(HMONITOR hMonitor, HDC hdc, LPRECT prc, LPARAM lParam);
   bool PassesFiltering(const CItemData &ci, const st_filters &filters);
   bool PassesPWHFiltering(const CItemData *pci,
                           const st_filters &filters) const;
@@ -636,14 +683,14 @@ protected:
   afx_msg void OnSendEmail();
   afx_msg void OnCopyUsername();
   afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
-  afx_msg void OnListItemSelected(NMHDR *pNMHDR, LRESULT *pLResult);
-  afx_msg void OnTreeItemSelected(NMHDR *pNMHDR, LRESULT *pLResult);
-  afx_msg void OnKeydownItemlist(NMHDR *pNMHDR, LRESULT *pResult);
-  afx_msg void OnItemDoubleClick(NMHDR* pNotifyStruct, LRESULT* result);
-  afx_msg void OnHeaderRClick(NMHDR* pNotifyStruct, LRESULT* result);
-  afx_msg void OnHeaderNotify(NMHDR* pNotifyStruct, LRESULT* result);
-  afx_msg void OnHeaderBeginDrag(NMHDR* pNotifyStruct, LRESULT* result);
-  afx_msg void OnHeaderEndDrag(NMHDR* pNotifyStruct, LRESULT* result);
+  afx_msg void OnListItemSelected(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnTreeItemSelected(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnKeydownItemlist(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnItemDoubleClick(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnHeaderRClick(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnHeaderNotify(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnHeaderBeginDrag(NMHDR *pNotifyStruct, LRESULT *pLResult);
+  afx_msg void OnHeaderEndDrag(NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnCopyPassword();
   afx_msg void OnCopyPasswordMinimize();
   afx_msg void OnDisplayPswdSubset();
@@ -658,6 +705,7 @@ protected:
   afx_msg void OnMerge();
   afx_msg void OnCompare();
   afx_msg void OnSynchronize();
+  afx_msg void OnChangeMode();
   afx_msg void OnProperties();
   afx_msg void OnRestoreSafe();
   afx_msg void OnSaveAs();
@@ -665,20 +713,21 @@ protected:
   afx_msg void OnListView();
   afx_msg void OnTreeView();
   afx_msg void OnBackupSafe();
-  afx_msg void OnPasswordChange();
+  afx_msg void OnPassphraseChange();
   afx_msg void OnClearClipboard();
   afx_msg void OnDelete();
   afx_msg void OnEdit();
   afx_msg void OnRename();
   afx_msg void OnDuplicateEntry();
   afx_msg void OnOptions();
-  afx_msg void OnValidate();
+  afx_msg void OnManagePasswordPolicies();
   afx_msg void OnGeneratePassword();
   afx_msg void OnSave();
   afx_msg void OnAdd();
   afx_msg void OnAddGroup();
   afx_msg void OnDuplicateGroup();
   afx_msg void OnProtect(UINT nID);
+  afx_msg void OnCompareEntries();
   afx_msg void OnCreateShortcut();
   afx_msg void OnOK();
   afx_msg void OnShowHideToolbar();
@@ -691,9 +740,10 @@ protected:
   afx_msg void OnChangeTreeFont();
   afx_msg void OnChangePswdFont();
   afx_msg void OnChangeVKFont();
-  afx_msg void OnViewReports(UINT nID);  // From View->Reports menu
+  afx_msg void OnViewReportsByID(UINT nID);  // From View->Reports menu
   afx_msg void OnViewReports();
   afx_msg void OnManageFilters(); // From Toolbar button
+  afx_msg void OnCancelFilter();
   afx_msg void OnApplyFilter();
   afx_msg void OnSetFilter();
   afx_msg void OnRefreshWindow();
@@ -715,20 +765,21 @@ protected:
 #else
   afx_msg void OnDropFiles(HDROP hDrop);
 #endif
-  afx_msg void OnColumnClick(NMHDR* pNMHDR, LRESULT* pResult);
+  afx_msg void OnColumnClick(NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnUpdateNSCommand(CCmdUI *pCmdUI);  // Make entry unsupported (grayed out)
   afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
   afx_msg void OnSizing(UINT fwSide, LPRECT pRect);
   //}}AFX_MSG
 
-  afx_msg BOOL OnToolTipText(UINT, NMHDR* pNMHDR, LRESULT* pResult);
+  afx_msg BOOL OnToolTipText(UINT, NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnExportVx(UINT nID);
   afx_msg void OnExportText();
   afx_msg void OnExportXML();
   afx_msg void OnExportEntryText();
   afx_msg void OnExportEntryXML();
   afx_msg void OnImportText();
-  afx_msg void OnImportKeePass();
+  afx_msg void OnImportKeePassV1CSV();
+  afx_msg void OnImportKeePassV1TXT();
   afx_msg void OnImportXML();
 
   afx_msg void OnToolBarFind();
@@ -757,21 +808,22 @@ protected:
 
 private:
   // UIInterFace implementations:
-  void DatabaseModified(bool bChanged);
-  void UpdateGUI(UpdateGUICommand::GUI_Action ga,
-                 uuid_array_t &entry_uuid,
-                 CItemData::FieldType ft, bool bUpdateGUI);
-  void GUISetupDisplayInfo(CItemData &ci);
-  void GUIRefreshEntry(const CItemData &ci);
-  void UpdateWizard(const stringT &s);
+  virtual void DatabaseModified(bool bChanged);
+  virtual void UpdateGUI(UpdateGUICommand::GUI_Action ga,
+                         const pws_os::CUUID &entry_uuid,
+                         CItemData::FieldType ft, bool bUpdateGUI);
+  virtual void GUISetupDisplayInfo(CItemData &ci);
+  virtual void GUIRefreshEntry(const CItemData &ci);
+  virtual void UpdateWizard(const stringT &s);
 
   static int CALLBACK CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
 
   // Only need these if not on default menus
-  static CString CS_SETFILTERS, CS_CLEARFILTERS;
+  static CString CS_SETFILTERS, CS_CLEARFILTERS,  CS_READWRITE, CS_READONLY;
 
   StringX m_BrowseURL; // set by OnContextMenu(), used by OnBrowse()
   PWScore &m_core;
+
   bool m_IsStartSilent;
   bool m_IsStartClosed;
   bool m_bStartHiddenAndMinimized;
@@ -787,13 +839,12 @@ private:
   int m_nColumnWidthByIndex[CItemData::LAST];
   int m_nColumnHeaderWidthByType[CItemData::LAST];
   int m_iheadermaxwidth;
-  CFont *m_pFontTree;
 
-  uuid_array_t m_LUUIDSelectedAtMinimize; // to restore List entry selection upon un-minimize
-  uuid_array_t m_TUUIDSelectedAtMinimize; // to restore Tree entry selection upon un-minimize
+  pws_os::CUUID m_LUUIDSelectedAtMinimize; // to restore List entry selection upon un-minimize
+  pws_os::CUUID m_TUUIDSelectedAtMinimize; // to restore Tree entry selection upon un-minimize
   StringX m_sxSelectedGroup;              // to restore Tree group selection upon un-minimize
-  uuid_array_t m_LUUIDVisibleAtMinimize;  // to restore List entry position  upon un-minimize
-  uuid_array_t m_TUUIDVisibleAtMinimize;  // to restore Tree entry position  upon un-minimize
+  pws_os::CUUID m_LUUIDVisibleAtMinimize;  // to restore List entry position  upon un-minimize
+  pws_os::CUUID m_TUUIDVisibleAtMinimize;  // to restore Tree entry position  upon un-minimize
   StringX m_sxVisibleGroup;               // to restore Tree group position  upon un-minimize
 
   bool m_inExit; // help U3ExitNow
@@ -818,7 +869,7 @@ private:
   void CheckExpireList(const bool bAtOpen = false); // Upon open, timer + menu, check list, show exp.
   void TellUserAboutExpiredPasswords();
   bool RestoreWindowsData(bool bUpdateWindows, bool bShow = true);
-  void UpdateAccessTime(CItemData *pci);
+  void UpdateAccessTime(const pws_os::CUUID &uuid);
   void RestoreGroupDisplayState();
   std::vector<bool> GetGroupDisplayState(); // get current display state from window
   void SetGroupDisplayState(const std::vector<bool> &displaystatus); // changes display
@@ -831,14 +882,14 @@ private:
   void RegistryAnonymity();
   void CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID, const bool bDoShortcuts);
   void SetUpMenuStrings(CMenu *pPopupMenu);
-  void SetUpInitialMenuStrings(LCID lcid = 0);
+  void SetUpInitialMenuStrings();
   void UpdateAccelTable();
   void SetupSpecialShortcuts();
   bool ProcessLanguageMenu(CMenu *pPopupMenu);
   void DoBrowse(const bool bDoAutotype, const bool bSendEmail);
   bool GetSubtreeEntriesProtectedStatus(int &numProtected, int &numUnprotected);
   void ChangeSubtreeEntriesProtectStatus(const UINT nID);
-  void CopyDataToClipBoard(const CItemData::FieldType ft, const bool special = false);
+  void CopyDataToClipBoard(const CItemData::FieldType ft, const bool bSpecial = false);
   void UpdateSystemMenu();
   void RestoreWindows(); // extended ShowWindow(SW_RESTORE), sort of
   void CancelPendingPasswordDialog();
@@ -893,9 +944,6 @@ private:
   // Menu Shortcuts
   MapMenuShortcuts m_MapMenuShortcuts;
 
-  // Mapping of virtual key to key name in user's locale
-  MapKeyNameID m_MapKeyNameID;
-
   // Menu items we don't allow the user to modify or see in Options
   // Shortcuts CListCtrl
   // Any popup menu and Exit and Help + a few special one
@@ -961,26 +1009,20 @@ private:
   // Might need to add more e.g. if filter is active and which one?
   struct st_SaveGUIInfo {
     bool blSelectedValid, btSelectedValid, btGroupValid;
-    uuid_array_t lSelected; // List selected item
-    uuid_array_t tSelected; // Tree selected item
+    pws_os::CUUID lSelected; // List selected item
+    pws_os::CUUID tSelected; // Tree selected item
     StringX sxGroupName;
     std::vector<bool> vGroupDisplayState;
 
     st_SaveGUIInfo()
-    : blSelectedValid(false), btSelectedValid(false), btGroupValid(false)
-    {
-      memset((void *)lSelected, 0, sizeof(uuid_array_t));
-      memset((void *)tSelected, 0, sizeof(uuid_array_t));
-    }
+    : blSelectedValid(false), btSelectedValid(false), btGroupValid(false),
+      lSelected(pws_os::CUUID::NullUUID()), tSelected(pws_os::CUUID::NullUUID()) {}
 
     st_SaveGUIInfo(const st_SaveGUIInfo &that)
     : blSelectedValid(that.blSelectedValid), btSelectedValid(that.btSelectedValid),
       btGroupValid(that.btGroupValid), vGroupDisplayState(that.vGroupDisplayState),
-      sxGroupName(that.sxGroupName)
-    {
-      memcpy((void *)lSelected, (void *)that.lSelected, sizeof(uuid_array_t));
-      memcpy((void *)tSelected, (void *)that.tSelected, sizeof(uuid_array_t));
-    }
+      lSelected(that.lSelected), tSelected(that.tSelected),
+      sxGroupName(that.sxGroupName) {}
 
     st_SaveGUIInfo &operator=(const st_SaveGUIInfo &that)
     {
@@ -988,8 +1030,8 @@ private:
         blSelectedValid = that.blSelectedValid;
         btSelectedValid = that.blSelectedValid;
         btGroupValid = that.btGroupValid;
-        memcpy((void *)lSelected, (void *)that.lSelected, sizeof(uuid_array_t));
-        memcpy((void *)tSelected, (void *)that.tSelected, sizeof(uuid_array_t));
+        lSelected = that.lSelected;
+        tSelected = that.tSelected;
         sxGroupName = that.sxGroupName;
         vGroupDisplayState = that.vGroupDisplayState;
       }
