@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -32,7 +32,7 @@ static char THIS_FILE[] = __FILE__;
 // Functor for count_if
 struct CountShortcuts {
   bool operator()(const std::pair<const UINT, CMenuShortcut> &p) {
-    return (p.second.cVirtKey != (unsigned char)0);
+    return (p.second.siVirtKey != 0);
   }
 };
 
@@ -40,14 +40,15 @@ struct CountShortcuts {
 struct CreateAccelTable {
   CreateAccelTable(ACCEL *pacceltbl, unsigned char ucAutotypeKey) 
     : m_pacceltbl(pacceltbl), m_ucAutotypeKey(ucAutotypeKey) {}
+
   void operator()(const std::pair<UINT, CMenuShortcut> &p)
   {
-    if (p.second.cVirtKey != 0 && p.second.cVirtKey != m_ucAutotypeKey) {
+    if (p.second.siVirtKey != 0 && p.second.siVirtKey != m_ucAutotypeKey) {
       m_pacceltbl->fVirt = FVIRTKEY |
-                           ((p.second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL ? FCONTROL : 0) |
-                           ((p.second.cModifier & HOTKEYF_ALT)    == HOTKEYF_ALT      ? FALT     : 0) |
-                           ((p.second.cModifier & HOTKEYF_SHIFT)  == HOTKEYF_SHIFT    ? FSHIFT   : 0);
-      m_pacceltbl->key = (WORD)p.second.cVirtKey;
+               ((p.second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL ? FCONTROL : 0) |
+               ((p.second.cModifier & HOTKEYF_ALT)     == HOTKEYF_ALT     ? FALT     : 0) |
+               ((p.second.cModifier & HOTKEYF_SHIFT)   == HOTKEYF_SHIFT   ? FSHIFT   : 0);
+      m_pacceltbl->key = (WORD)p.second.siVirtKey;
       m_pacceltbl->cmd = (WORD)p.first;
       m_pacceltbl++;
     }
@@ -132,7 +133,7 @@ void InsertShortcuts(CMenu *pMenu, MapMenuShortcuts &mms,
   }
 }
 
-void DboxMain::SetUpInitialMenuStrings(LCID lcid)
+void DboxMain::SetUpInitialMenuStrings()
 {
   CMenu *pMainMenu, *pSubMenu;
   int isubmenu_pos;
@@ -140,9 +141,6 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
 
   CHotKeyCtrl cHKC;
   CString sKeyName;
-  LPCWSTR ptcKeyName;
-  wchar_t *pname;
-  st_KeyIDExt st_KIDEx;
   BOOL brc;
 
   // Following are excluded from list of user-configurable
@@ -175,81 +173,6 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
   m_ExcludedMenuItems.assign(excludedMenuItems,
                              excludedMenuItems + _countof(excludedMenuItems));
 
-  std::pair< MapKeyNameIDIter, bool > prMKNID;
-
-  // Clear out any there
-  MapKeyNameIDIter KNIDiter;
-  for (KNIDiter = m_MapKeyNameID.begin(); KNIDiter != m_MapKeyNameID.end(); KNIDiter++) {
-    free((void *)KNIDiter->second);
-    KNIDiter->second = NULL;
-  }
-  m_MapKeyNameID.clear();
-
-  // Add in the zero/None entry
-  pname = _wcsdup(L"");
-  st_KIDEx.id = 0;
-  st_KIDEx.bExtended = false;
-  prMKNID = m_MapKeyNameID.insert(MapKeyNameIDPair(st_KIDEx, pname));
-  ASSERT(prMKNID.second == true);
-
-  HKL hkl(NULL), prevhkl(NULL);
-  if (lcid != 0) {
-    // This won't work for Japanese, Korean or Chinese as they use IMEs and not keyboard layouts :-(
-    CString csKLID;
-    csKLID.Format(L"%08x", lcid);
-
-    // Load new keyboard layout
-    hkl = LoadKeyboardLayout(csKLID, KLF_NOTELLSHELL);
-    // Activate it so that we get the correct names for things like "Scroll-Lock", "Nackspace" etc.
-    prevhkl = ActivateKeyboardLayout(hkl, KLF_SETFORPROCESS);
-  }
-
-  // Now add in locale key names (note range 0xE0-0xFF not used)
-  for (int i = 1; i < 0xE0; i++) {
-    // Following are reserved or unassigned by Windows
-    if ( i == 0x07 || i == 0x0A  || 
-         i == 0x0B || i == 0x1B  ||
-        (i >= 0x3A && i <= 0x40) ||
-        (i >= 0x5B && i <= 0x5F) ||
-        (i >= 0x88 && i <= 0x8F) ||
-        (i >= 0x92 && i <= 0x9F) ||
-        (i >= 0xA6 && i <= 0xB9) ||
-        (i >= 0xC1 && i <= 0xDA))
-      continue;
-
-    st_KIDEx.id = (unsigned char)i;
-    st_KIDEx.bExtended = IsExtended(i);
-    sKeyName = cHKC.GetKeyName((UINT)i, IsExtended(i) ? TRUE : FALSE);
-    if (!sKeyName.IsEmpty()) {
-      // Make value into "Sentence Case" e.g. "Enter" or "Num Plus" etc.
-      CString cstoken, sKeyName2(L"");
-      int curPos = 0;
-      sKeyName.Trim();
-      cstoken = sKeyName.Tokenize(L" ", curPos);
-      while (!cstoken.IsEmpty()) {
-        CString cstemp1 = cstoken.Left(1);
-        CString cstemp2 = cstoken.Right(cstoken.GetLength() - 1);
-        cstemp1.MakeUpper();
-        cstemp2.MakeLower();
-        sKeyName2 += cstemp1 + cstemp2 + CString(L" ");
-        cstoken = sKeyName.Tokenize(L" ", curPos);
-      };
-      sKeyName2.Trim();
-      ptcKeyName = sKeyName2.GetBuffer(sKeyName2.GetLength());
-      pname = _wcsdup(ptcKeyName);
-      sKeyName2.ReleaseBuffer();
-      prMKNID = m_MapKeyNameID.insert(MapKeyNameIDPair(st_KIDEx, pname));
-      ASSERT(prMKNID.second == true);
-    }
-  }
-
-  if (hkl != NULL) {
-    // Put keyboard layout back as it was.
-    UnloadKeyboardLayout(hkl);
-    if (prevhkl != NULL)
-      ActivateKeyboardLayout(prevhkl, KLF_SETFORPROCESS);
-  }
-
   pMainMenu = new CMenu;
   brc = pMainMenu->LoadMenu(IDR_MAINMENU);
   ASSERT(brc != 0);
@@ -266,7 +189,7 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
   m_ReservedShortcuts.clear();
 
   st_MenuShortcut st_mst;
-  st_mst.cVirtKey;
+  st_mst.siVirtKey;
   st_mst.cModifier = HOTKEYF_ALT;
 
   for (UINT ui = 0; ui < uiCount; ui++) {
@@ -285,22 +208,22 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
       CString csMainMenuItem = tcMenuString;
       int iamp = csMainMenuItem.Find(L'&');
       if (iamp >= 0 && iamp < csMainMenuItem.GetLength() - 1) {
-        st_mst.cVirtKey = (unsigned char)csMainMenuItem[iamp + 1];
+        st_mst.siVirtKey = csMainMenuItem[iamp + 1];
         m_ReservedShortcuts.push_back(st_mst);
       }
     }
   }
 
   // Add 3 special keys F1 (for Help), Ctrl+Q/Alt+F4 (Exit)
-  st_mst.cVirtKey = VK_F1;
+  st_mst.siVirtKey = VK_F1;
   st_mst.cModifier = 0;
   m_ReservedShortcuts.push_back(st_mst);
 
-  st_mst.cVirtKey = 'Q';
+  st_mst.siVirtKey = 'Q';
   st_mst.cModifier = HOTKEYF_CONTROL;
   m_ReservedShortcuts.push_back(st_mst);
 
-  st_mst.cVirtKey = VK_F4;
+  st_mst.siVirtKey = VK_F4;
   st_mst.cModifier = HOTKEYF_ALT;
   m_ReservedShortcuts.push_back(st_mst);
 
@@ -404,9 +327,9 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
 
       iter = m_MapMenuShortcuts.find((UINT)paccel->cmd);
       if (iter != m_MapMenuShortcuts.end()) {
-        iter->second.cdefVirtKey = iter->second.cVirtKey = 
+        iter->second.siDefVirtKey = iter->second.siVirtKey = 
           (unsigned char)paccel->key;
-          iter->second.cdefModifier = iter->second.cModifier =
+          iter->second.cDefModifier = iter->second.cModifier =
           ((paccel->fVirt & FCONTROL) == FCONTROL ? HOTKEYF_CONTROL : 0) |
           ((paccel->fVirt & FALT)     == FALT     ? HOTKEYF_ALT     : 0) |
           ((paccel->fVirt & FSHIFT)   == FSHIFT   ? HOTKEYF_SHIFT   : 0) |
@@ -424,6 +347,20 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
   // change shortcuts as per preferences
   std::vector<st_prefShortcut> vShortcuts(PWSprefs::GetInstance()->GetPrefShortcuts());
 
+  // We need to convert from PWS shortcut modifiers to MFC modifiers
+  for (size_t i = 0; i < vShortcuts.size(); i++) {
+    unsigned char cModifier(0), cWindowsMod = vShortcuts[i].cModifier;
+    if ((cWindowsMod & PWS_HOTKEYF_ALT    ) == PWS_HOTKEYF_ALT)
+      cModifier |= HOTKEYF_ALT;
+    if ((cWindowsMod & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL)
+      cModifier |= HOTKEYF_CONTROL;
+    if ((cWindowsMod & PWS_HOTKEYF_SHIFT  ) == PWS_HOTKEYF_SHIFT)
+      cModifier |= HOTKEYF_SHIFT;
+    if ((cWindowsMod & PWS_HOTKEYF_EXT    ) == PWS_HOTKEYF_EXT)
+      cModifier |= HOTKEYF_EXT;
+    vShortcuts[i].cModifier = cModifier;
+  }
+
   size_t N = vShortcuts.size();
   for (size_t i = 0; i < N; i++) {
     const st_prefShortcut &stxst = vShortcuts[i];
@@ -443,8 +380,8 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
       continue;
     }
     // Check not already in use (ignore if deleting current shortcut)
-    if (stxst.cVirtKey != (unsigned char)0) {
-      st_mst.cVirtKey = stxst.cVirtKey;
+    if (stxst.siVirtKey != 0) {
+      st_mst.siVirtKey = stxst.siVirtKey;
       st_mst.cModifier = stxst.cModifier;
       already_inuse inuse(st_mst);
       inuse_iter = std::find_if(m_MapMenuShortcuts.begin(),
@@ -456,10 +393,10 @@ void DboxMain::SetUpInitialMenuStrings(LCID lcid)
         continue;
       }
     }
-    if ((iter->second.cVirtKey  != stxst.cVirtKey  ||
+    if ((iter->second.siVirtKey  != stxst.siVirtKey  ||
          iter->second.cModifier != stxst.cModifier)) {
       // User changed a added or shortcut
-      iter->second.cVirtKey  = stxst.cVirtKey;
+      iter->second.siVirtKey  = stxst.siVirtKey;
       iter->second.cModifier = stxst.cModifier;
     }
   }
@@ -528,8 +465,6 @@ void DboxMain::SetUpMenuStrings(CMenu *pPopupMenu)
   MENUITEMINFO miteminfo = {0};
 
   MapMenuShortcutsIter iter;
-  MapKeyNameIDConstIter citer;
-  st_KeyIDExt st_KIDEx;
 
   UINT uiCount = pPopupMenu->GetMenuItemCount();
   ASSERT((int)uiCount >= 0);
@@ -548,16 +483,9 @@ void DboxMain::SetUpMenuStrings(CMenu *pPopupMenu)
       iter = m_MapMenuShortcuts.find(miteminfo.wID);
       if (iter != m_MapMenuShortcuts.end()) {
         CString str;
-        if (iter->second.cVirtKey != 0) {
-          st_KIDEx.id = iter->second.cVirtKey;
-          st_KIDEx.bExtended = (iter->second.cModifier & HOTKEYF_EXT) == HOTKEYF_EXT;
-          citer = m_MapKeyNameID.find(st_KIDEx);
-          if (citer != m_MapKeyNameID.end()) {
-            str.Format(L"%s\t%s", iter->second.name.c_str(), 
-                       CMenuShortcut::FormatShortcut(iter, citer));
-          } else
-            continue;
-
+        if (iter->second.siVirtKey != 0) {
+          str.Format(L"%s\t%s", iter->second.name.c_str(), 
+                       CMenuShortcut::FormatShortcut(iter));
         } else {
           str = iter->second.name.c_str();
         }
@@ -591,15 +519,6 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
   if (bDoShortcuts)
     SetUpMenuStrings(pPopupMenu);
 
-  // We have done for all except the Edit and View menus
-  if (uiMenuID != ID_FILEMENU && uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU)
-    return;
-
-  if (bItemSelected) {
-    pci = getSelectedItem();
-    ASSERT(pci != NULL);
-  }
-
 	// Change the 'Change Mode' text as appropriate	 
   if (uiMenuID == ID_FILEMENU) {	 
     pPopupMenu->ModifyMenu(ID_MENUITEM_CHANGEMODE, MF_BYCOMMAND,	 
@@ -608,6 +527,10 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     return;	 
   }
 
+  // We have done for all except the Edit and View menus
+  if (uiMenuID != ID_EDITMENU && uiMenuID != ID_VIEWMENU && uiMenuID != ID_FILTERMENU)
+    return;
+
   // If View menu selected (contains 'Flattened &List' menu item)
   if (uiMenuID == ID_VIEWMENU) {
     // Delete Show Find Toolbar menu item - don't know if there or previously deleted
@@ -636,14 +559,10 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     pPopupMenu->CheckMenuItem(ID_MENUITEM_SHOW_ALL_EXPIRY, MF_BYCOMMAND |
                               m_bExpireDisplayed ? MF_CHECKED : MF_UNCHECKED);
 
+    // Don't show filter menu if "internal" menu active
     pPopupMenu->EnableMenuItem(ID_FILTERMENU, MF_BYCOMMAND |
              (m_bUnsavedDisplayed || m_bExpireDisplayed) ? MF_GRAYED : MF_ENABLED);
 
-    pPopupMenu->ModifyMenu(ID_MENUITEM_APPLYFILTER, MF_BYCOMMAND |
-                           m_bFilterActive ? MF_CHECKED : MF_UNCHECKED,
-                           ID_MENUITEM_APPLYFILTER,
-                           m_bFilterActive ? CS_CLEARFILTERS : CS_SETFILTERS);
-
     pPopupMenu->CheckMenuRadioItem(ID_MENUITEM_NEW_TOOLBAR,
                                    ID_MENUITEM_OLD_TOOLBAR,
                                    m_toolbarMode, MF_BYCOMMAND);
@@ -654,6 +573,18 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     goto exit;
   }  // View menu
 
+  if (uiMenuID == ID_FILTERMENU) {
+    pPopupMenu->ModifyMenu(1, MF_BYPOSITION,
+                           m_bFilterActive ? ID_MENUITEM_CLEARFILTER : ID_MENUITEM_APPLYFILTER,
+                           m_bFilterActive ? CS_CLEARFILTERS : CS_SETFILTERS);
+    goto exit;
+  }
+
+  if (bItemSelected) {
+    pci = getSelectedItem();
+    ASSERT(pci != NULL);
+  }
+
   // Save original entry type before possibly changing pci
   const CItemData::EntryType etype_original = 
           pci == NULL ? CItemData::ET_INVALID : pci->GetEntryType();
@@ -712,11 +643,13 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
           pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
         }
       }
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_UNDO, tc_dummy);
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_REDO, tc_dummy);
-      pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_UNDO, tc_dummy);
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_REDO, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       goto exit;
@@ -753,11 +686,13 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         }
       }
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_UNDO, tc_dummy);
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_REDO, tc_dummy);
-      pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_UNDO, tc_dummy);
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_REDO, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       goto exit;
@@ -770,6 +705,21 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
     //               Entry functions (copy to clipboard, browse, run etc)
     if (pci != NULL) {
       // Entry is selected
+
+      // Deal with multi-selection
+      // More than 2 is meaningless in List view
+      if (m_IsListView && m_ctlItemList.GetSelectedCount() > 2)
+        return;
+
+      // If exactly 2 selected - show compare entries menu
+      if (m_IsListView  && m_ctlItemList.GetSelectedCount() == 2) {
+        CString cs_txt(MAKEINTRESOURCE(ID_MENUITEM_COMPARE_ENTRIES));
+        cs_txt.TrimLeft();  // Remove leading newline
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_COMPARE_ENTRIES, cs_txt);
+        return;
+      }
+
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_ADD, tc_dummy);
@@ -799,18 +749,20 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_DUPLICATEENTRY, tc_dummy);
-        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
         if (!m_IsListView) {
+          pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
           pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                  ID_MENUITEM_ADDGROUP, tc_dummy);
         }
       }
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_UNDO, tc_dummy);
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_REDO, tc_dummy);
-      pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_UNDO, tc_dummy);
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_REDO, tc_dummy);
+        pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      }
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
@@ -905,6 +857,12 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                pci->IsProtected() ? ID_MENUITEM_UNPROTECT : ID_MENUITEM_PROTECT,
                                tc_dummy);
+
+      // Tree view and command flag present only
+      if (!m_IsListView && m_bCompareEntries &&
+           etype_original != CItemData::ET_SHORTCUT)
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_COMPARE_ENTRIES, tc_dummy);
     } else {
       // Must be List view with no entry selected
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
@@ -945,44 +903,16 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
   bool bDoShortcuts(false);
   switch (minfo.dwMenuData) {
     case ID_FILEMENU:
-      bDoShortcuts = m_bDoShortcuts[FILEMENU];
-      m_bDoShortcuts[FILEMENU] = false;
-      break;
     case ID_EXPORTMENU:
-      bDoShortcuts = m_bDoShortcuts[EXPORTMENU];
-      m_bDoShortcuts[EXPORTMENU] = false;
-      break;
     case ID_IMPORTMENU:
-      bDoShortcuts = m_bDoShortcuts[IMPORTMENU];
-      m_bDoShortcuts[IMPORTMENU] = false;
-      break;
     case ID_EDITMENU:
-      bDoShortcuts = m_bDoShortcuts[EDITMENU];
-      m_bDoShortcuts[EDITMENU] = false;
-      break;
     case ID_VIEWMENU:
-      bDoShortcuts = m_bDoShortcuts[VIEWMENU];
-      m_bDoShortcuts[VIEWMENU] = false;
-      break;
     case ID_FILTERMENU:
-      bDoShortcuts = m_bDoShortcuts[FILTERMENU];
-      m_bDoShortcuts[FILTERMENU] = false;
-      break;
     case ID_CHANGEFONTMENU:
-      bDoShortcuts = m_bDoShortcuts[CHANGEFONTMENU];
-      m_bDoShortcuts[CHANGEFONTMENU] = false;
-      break;
     case ID_REPORTSMENU:
-      bDoShortcuts = m_bDoShortcuts[REPORTSMENU];
-      m_bDoShortcuts[REPORTSMENU] = false;
-      break;
     case ID_MANAGEMENU:
-      bDoShortcuts = m_bDoShortcuts[MANAGEMENU];
-      m_bDoShortcuts[MANAGEMENU] = false;
-      break;
-    case ID_HELPMENU:
-      bDoShortcuts = m_bDoShortcuts[HELPMENU];
-      m_bDoShortcuts[HELPMENU] = false;
+    case ID_HELPMENU:  //main menu items' shortuct should be always updated because keyboard layout could be changed
+      bDoShortcuts = true;
       break;
     default:
       break;
@@ -1127,7 +1057,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
 }
 
 // Called when right-click is invoked in the client area of the window.
-void DboxMain::OnContextMenu(CWnd* /* pWnd */, CPoint screen)
+void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
 {
 #if defined(POCKET_PC)
   const DWORD dwTrackPopupFlags = TPM_LEFTALIGN;
@@ -1250,7 +1180,7 @@ void DboxMain::OnContextMenu(CWnd* /* pWnd */, CPoint screen)
         item = pdi->list_index;
         m_ctlItemTree.SelectItem(ti); // So that OnEdit gets the right one
       } else {
-        // right-click was on a group (NODE)
+        // right-click was on a group (GROUP)
         m_ctlItemTree.SelectItem(ti);
         if (menu.LoadMenu(IDR_POPEDITGROUP)) {
           minfo.dwMenuData = IDR_POPEDITGROUP;
@@ -1295,6 +1225,29 @@ void DboxMain::OnContextMenu(CWnd* /* pWnd */, CPoint screen)
 
   // RClick over an entry
   if (item >= 0) {
+    CMenu *pPopup(NULL);
+
+    // More than 2 is meaningless in List view
+    if (m_IsListView && m_ctlItemList.GetSelectedCount() > 2)
+      return;
+
+    // Only compare entries if 2 entries are selected - List view only
+    if (m_IsListView && m_ctlItemList.GetSelectedCount() == 2) {
+      brc = menu.LoadMenu(IDR_POPCOMPAREENTRIES);
+      ASSERT(brc != 0);
+
+      minfo.dwMenuData = IDR_POPCOMPAREENTRIES;
+      brc = menu.SetMenuInfo(&minfo);
+      ASSERT(brc != 0);
+
+      pPopup = menu.GetSubMenu(0);
+      ASSERT_VALID(pPopup);
+
+      // Use this DboxMain for commands
+      pPopup->TrackPopupMenu(dwTrackPopupFlags, screen.x, screen.y, this);
+      return;
+    }
+
     ASSERT(pci != NULL);
     brc = menu.LoadMenu(IDR_POPEDITMENU);
     ASSERT(brc != 0);
@@ -1303,7 +1256,7 @@ void DboxMain::OnContextMenu(CWnd* /* pWnd */, CPoint screen)
     brc = menu.SetMenuInfo(&minfo);
     ASSERT(brc != 0);
 
-    CMenu* pPopup = menu.GetSubMenu(0);
+    pPopup = menu.GetSubMenu(0);
     ASSERT_VALID(pPopup);
 
     const CItemData::EntryType etype_original = pci->GetEntryType();
@@ -1367,9 +1320,19 @@ void DboxMain::OnContextMenu(CWnd* /* pWnd */, CPoint screen)
       pPopup->RemoveMenu(ID_MENUITEM_RUNCOMMAND, MF_BYCOMMAND);
     }
 
+    if (m_IsListView) {
+      // Rename not valid in List View
+      pPopup->RemoveMenu(ID_MENUITEM_RENAMEENTRY, MF_BYCOMMAND);
+    }
+
     // Since an entry - remove Duplicate Group
     pPopup->RemoveMenu(ID_MENUITEM_DUPLICATEGROUP, MF_BYCOMMAND);
 
+    // Remove if in List View, not allowed in Tree view or a shortcut
+    if (m_IsListView || !m_bCompareEntries ||
+        etype_original == CItemData::ET_SHORTCUT)
+        pPopup->RemoveMenu(ID_MENUITEM_COMPARE_ENTRIES, MF_BYCOMMAND);
+
     // Use this DboxMain for commands
     pPopup->TrackPopupMenu(dwTrackPopupFlags, screen.x, screen.y, this);
   } // if (item >= 0)
@@ -1389,9 +1352,10 @@ void DboxMain::SetupSpecialShortcuts()
     iter_group = m_MapMenuShortcuts.find(ID_MENUITEM_DELETEGROUP);
     iter_group->second.SetKeyFlags(iter->second);
 
-    m_wpDeleteMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ? WM_SYSKEYDOWN : WM_KEYDOWN;
-    m_wpDeleteKey = iter->second.cVirtKey;
-    m_bDeleteCtrl = (iter->second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
+    m_wpDeleteMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+                           WM_SYSKEYDOWN : WM_KEYDOWN;
+    m_wpDeleteKey = iter->second.siVirtKey;
+    m_bDeleteCtrl = (iter->second.cModifier  & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
     m_bDeleteShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
   } else {
     m_wpDeleteKey = 0;
@@ -1407,8 +1371,9 @@ void DboxMain::SetupSpecialShortcuts()
     iter_group = m_MapMenuShortcuts.find(ID_MENUITEM_RENAMEGROUP);
     iter_group->second.SetKeyFlags(iter->second);
 
-    m_wpRenameMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ? WM_SYSKEYDOWN : WM_KEYDOWN;
-    m_wpRenameKey = iter->second.cVirtKey;
+    m_wpRenameMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+                           WM_SYSKEYDOWN : WM_KEYDOWN;
+    m_wpRenameKey = iter->second.siVirtKey;
     m_bRenameCtrl = (iter->second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
     m_bRenameShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
   } else {
@@ -1420,9 +1385,11 @@ void DboxMain::SetupSpecialShortcuts()
   
   // Save for CTreeCtrl & CListCtrl PreTranslateMessage
   if (iter != m_MapMenuShortcuts.end()) {
-    m_wpAutotypeDNMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ? WM_SYSKEYDOWN : WM_KEYDOWN;
-    m_wpAutotypeUPMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ? WM_SYSKEYUP : WM_KEYUP;
-    m_wpAutotypeKey = iter->second.cVirtKey;
+    m_wpAutotypeDNMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+                               WM_SYSKEYDOWN : WM_KEYDOWN;
+    m_wpAutotypeUPMsg = ((iter->second.cModifier & HOTKEYF_ALT) == HOTKEYF_ALT) ?
+                               WM_SYSKEYUP : WM_KEYUP;
+    m_wpAutotypeKey = iter->second.siVirtKey;
     m_bAutotypeCtrl = (iter->second.cModifier & HOTKEYF_CONTROL) == HOTKEYF_CONTROL;
     m_bAutotypeShift = (iter->second.cModifier & HOTKEYF_SHIFT) == HOTKEYF_SHIFT;
   } else {
