@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,7 +17,6 @@
 
 #include "PasswordSafe.h"
 
-
 #if defined(POCKET_PC)
 #include "pocketpc/PocketPC.h"
 #include "pocketpc/resource.h"
@@ -34,6 +33,9 @@
 #include "PWSRecentFileList.h"
 #include "MFCMessages.h"
 #include "GeneralMsgBox.h"
+#include "PWSFaultHandler.h"
+#include "Fonts.h"
+#include "PWSversion.h"
 
 #include "core/Util.h"
 #include "core/BlowFish.h"
@@ -41,7 +43,7 @@
 #include "core/PWSrand.h"
 #include "core/PWSdirs.h"
 #include "core/SysInfo.h"
-#include "core/XMLprefs.h"
+#include "core/PWSLog.h"
 
 #include "os/windows/pws_autotype/pws_at.h"
 #include "os/dir.h"
@@ -57,10 +59,6 @@
 // Only produce minidumps in release code
 #ifndef _DEBUG
 #include "Dbghelp.h"
-void InstallFaultHandler(const int major, const int minor, const int build,
-                         const TCHAR * revision, const DWORD dwTimeStamp);
-void LocalizeFaultHandler(HINSTANCE inst);
-void RemoveFaultHandler(bool bFreeLibrary = true);
 #endif
 
 using namespace std;
@@ -95,7 +93,7 @@ ThisMfcApp::ThisMfcApp() :
 #else
   m_bUseAccelerator(true),
 #endif
-  m_pMRU(NULL), m_TrayLockedState(LOCKED), m_TrayIcon(NULL),
+  m_pMRU(NULL), m_TrayLockedState(LOCKED), m_pTrayIcon(NULL),
   m_HotKeyPressed(false), m_hMutexOneInstance(NULL),
   m_ghAccelTable(NULL), m_pMainMenu(NULL),
   m_bACCEL_Table_Created(false), m_noSysEnvWarnings(false),
@@ -173,7 +171,6 @@ ThisMfcApp::ThisMfcApp() :
 
 ThisMfcApp::~ThisMfcApp()
 {
-  delete m_TrayIcon;
   delete m_pMRU;
 
   // Remove MRU menu
@@ -198,6 +195,10 @@ ThisMfcApp::~ThisMfcApp()
 
   PWSprefs::DeleteInstance();
   PWSrand::DeleteInstance();
+  PWSversion::DeleteInstance();
+  Fonts::DeleteInstance();
+  PWSLog::DeleteLog();
+
   CoUninitialize(); // Uninitialize COM library
 
 #if !defined(POCKET_PC)
@@ -238,8 +239,22 @@ static bool CheckFile(const CString &fn)
     return false;
   }
 }
-
 #endif // !POCKET_PC
+
+void ThisMfcApp::SetMinidumpUserStreams(const bool bOpen, const bool bRW, UserStream iStream)
+{
+#ifndef _DEBUG
+  PWSprefs *prefs = PWSprefs::GetInstance();
+  if (prefs != NULL) {
+    PopulateMinidumpUserStreams(prefs, bOpen, bRW, iStream);
+  }
+#else
+  UNREFERENCED_PARAMETER(bOpen);
+  UNREFERENCED_PARAMETER(bRW);
+  UNREFERENCED_PARAMETER(iStream);
+#endif
+}
+
 int ThisMfcApp::ExitInstance()
 {
   if (m_hInstResDLL != NULL && m_hInstResDLL != m_hInstance)
@@ -327,8 +342,10 @@ void ThisMfcApp::LoadLocalizedStuff()
   defaulting to pwsafe.chm if not found.
   */
 
-  CString cs_LANG, cs_CTRY(L"");
-
+  CString cs_ResPath, cs_HelpPath, cs_LANG(L""), cs_CTRY(L"");
+  const CString cs_HelpDir(PWSdirs::GetHelpDir().c_str());
+  const CString cs_ExePath(PWSdirs::GetExeDir().c_str());
+  
   // See if user overridden default via Environmental Variable
   bool bPLRC(FALSE);
   
@@ -346,8 +363,32 @@ void ThisMfcApp::LoadLocalizedStuff()
       cs_CTRY = sxLL.substr(3,2).c_str();
     }
     bPLRC = true;
-  }    
-    
+  }
+
+  if (cs_LANG == L"EN" && cs_CTRY.IsEmpty()) {
+    // This means use the embedded resources!
+    if (m_hInstResDLL)
+      FreeLibrary(m_hInstResDLL);
+    m_hInstResDLL = NULL;
+    pws_os::Trace(L"Using embedded resources\n");
+
+    //  and supplied Help file
+    free((void *)m_pszHelpFilePath);
+
+    cs_HelpPath.Format(L"%spwsafe.chm", cs_HelpDir);
+    if (PathFileExists(cs_HelpPath)) {
+      m_pszHelpFilePath = _wcsdup(cs_HelpPath);
+      m_csHelpFile = cs_HelpPath;
+      pws_os::Trace(L"Using help file: %s\n", cs_HelpPath);
+    } else {
+      m_pszHelpFilePath = NULL;
+      m_csHelpFile = L"";
+      pws_os::Trace(L"Not using a help file!\n");
+    }
+
+    return;
+  }
+
   if (!bPLRC) {
     // no override, use Locale info
     int inum;
@@ -363,8 +404,7 @@ void ThisMfcApp::LoadLocalizedStuff()
     cs_LANG = szLang; cs_CTRY = szCtry;
   }
 
-  const CString cs_ExePath(PWSdirs::GetExeDir().c_str());
-  CString cs_ResPath;
+  // Find reseource-only DLL if requested
   const CString format_string = (cs_CTRY.IsEmpty()) ?
                       L"%spwsafe%s%s.dll" : L"%spwsafe%s_%s.dll";
   cs_ResPath.Format(format_string, cs_ExePath, cs_LANG, cs_CTRY);
@@ -401,13 +441,7 @@ void ThisMfcApp::LoadLocalizedStuff()
   if (m_hInstResDLL != NULL)
     AfxSetResourceHandle(m_hInstResDLL);
 
-  /**
-  * So far, we've handle the resource dll. Now go for the compiled helpfile
-  * in a similar manner.
-  */
-
-  CString cs_HelpPath;
-  const CString cs_HelpDir(PWSdirs::GetHelpDir().c_str());
+  // Find requested Help file
   bool helpFileFound = false;
 
   if (!cs_LANG.IsEmpty() && !cs_CTRY.IsEmpty()) {
@@ -431,7 +465,7 @@ void ThisMfcApp::LoadLocalizedStuff()
     }
   }
 
-  if (!helpFileFound && _tcslen(m_pszHelpFilePath) > 0) { // last resort
+  if (!helpFileFound && m_pszHelpFilePath != NULL && _tcslen(m_pszHelpFilePath) > 0) { // last resort
     wchar_t fname[_MAX_FNAME];
     wchar_t ext[_MAX_EXT];
 #if (_MSC_VER >= 1400)
@@ -451,9 +485,8 @@ void ThisMfcApp::LoadLocalizedStuff()
     free((void*)m_pszHelpFilePath);
 
     m_pszHelpFilePath = _wcsdup(cs_HelpPath);
-    pws_os::Trace(L"Using help file: %s\n", cs_HelpPath);
-
     m_csHelpFile = cs_HelpPath;
+    pws_os::Trace(L"Using help file: %s\n", cs_HelpPath);
   }
 }
 
@@ -473,6 +506,11 @@ void ThisMfcApp::SetupMenu()
     m_pMainMenu = NULL;
   }
 
+  //Reload accelerators for current locale
+  if (app.m_ghAccelTable)
+    DestroyAcceleratorTable(app.m_ghAccelTable);
+  app.m_ghAccelTable=LoadAccelerators(AfxGetResourceHandle(), MAKEINTRESOURCE(IDR_ACCS));
+
   int nMRUItems = PWSprefs::GetInstance()->GetPref(PWSprefs::MaxMRUItems);
   m_mruonfilemenu = PWSprefs::GetInstance()->GetPref(PWSprefs::MRUOnFileMenu);
   m_pMainMenu = new CMenu;
@@ -494,10 +532,11 @@ void ThisMfcApp::SetupMenu()
 
   pMenu1 = m_pMainMenu->GetSubMenu(pos1);
   minfo.dwMenuData = ID_FILEMENU;
-  pMenu1->SetMenuInfo(&minfo);
-
-  if (pMenu1 != NULL) // Look for "Close Database"
+  
+  if (pMenu1 != NULL) { // Look for "Close Database"
+    pMenu1->SetMenuInfo(&minfo);
     pos2 = FindMenuItem(pMenu1, ID_MENUITEM_CLOSE);
+  }
   else
     pos2 = -1;
 
@@ -723,6 +762,18 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
            * State of m_bSetup is accessible via public IsSetup() member function
            */
           dbox.SetSetup();
+        } else if ((*arg) == L"--novalidate") {
+          /**
+           * '--novalidate' prevents SOME of validation during open
+           */
+          dbox.NoValidation();
+        } else if ((*arg) == L"--cetreeview") {
+          /**
+           * '--cetreeview' will allow the user to select 2 entries and compare them
+           * TEMPORARY solution until multi-select is coded in the main Tree view
+           * Supported natively in List View
+           */
+          dbox.AllowCompareEntries();
         } else {
           // unrecognized extended flag. Silently ignore.
         }
@@ -791,7 +842,7 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
           dbox.SetStartSilent(true);
           break;
         case L'V': case L'v':
-          dbox.SetValidate(true);
+          // Obsolete - databases are always validated during opening unless --novalidate specified
           break;
         case L'U': case L'u': // set effective user
           // ensure there's another non-flag argument
@@ -820,7 +871,19 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
             return FALSE;
           } else {
             arg++;
-            PWSprefs::SetConfigFile(std::wstring(*arg));
+            if (!PWSprefs::SetConfigFile(std::wstring(*arg))) {
+              // SetConfigFile returns false if specified file not found
+              CGeneralMsgBox gmb;
+              CString missing_cfg;
+              // Classic chicken-and-egg here: Since the local
+              // language could be specified in the config file, and since
+              // the desired config file can't be found, it seems
+              // safest to keep this particular error message in English
+              // and out of the language-specific dlls.
+              missing_cfg.Format(L"Configuration file %s not found - creating it.",
+                                 *arg);
+              gmb.AfxMessageBox(missing_cfg, L"Error", MB_OK|MB_ICONINFORMATION);
+            }
           }
           break;
         case L'Q': case L'q': // be Quiet re missing fonts, dlls, etc.
@@ -888,15 +951,12 @@ BOOL ThisMfcApp::InitInstance()
   // (since user/host/config file may be overriden!)
   bool allDone = false;
 
-  LoadLocalizedStuff();
-#ifndef _DEBUG
-  if (m_hInstResDLL != NULL)
-    LocalizeFaultHandler(m_hInstResDLL);
-#endif
   // Note: Even though PWSprefs has not yet been created, parsing the command line
   // will (via DboxMain) fill in values within it!
   bool parseVal = ParseCommandLine(dbox, allDone);
 
+  // allDone will be true iff -e or -d options given, in which case
+  // we're just a batch encryptor/decryptor
   if (allDone)
     return parseVal ? TRUE : FALSE;
   else if (!parseVal) // bad command line args
@@ -907,6 +967,20 @@ BOOL ThisMfcApp::InitInstance()
 
   PWSprefs *prefs = PWSprefs::GetInstance();
 
+  // And the others - even if not referenced here
+  Fonts *pFonts = Fonts::GetInstance();
+  PWSversion *pPWSver = PWSversion::GetInstance();
+
+  // Stop compiler warnings
+  UNREFERENCED_PARAMETER(pFonts);
+  UNREFERENCED_PARAMETER(pPWSver);
+
+  LoadLocalizedStuff();
+#ifndef _DEBUG
+  if (m_hInstResDLL != NULL)
+    LocalizeFaultHandler(m_hInstResDLL);
+#endif
+
   // Update Quiet value if via environmental variable rather than 
   // command line flag
   CString cs_PWS_QUIET;
@@ -990,16 +1064,10 @@ BOOL ThisMfcApp::InitInstance()
   m_UnLockedIcon = app.LoadIcon(IDI_UNLOCKEDICON);
   int iData = prefs->GetPref(PWSprefs::ClosedTrayIconColour);
   SetClosedTrayIcon(iData);
-  m_TrayIcon = new CSystemTray((CWnd *)m_pMainDlg, PWS_MSG_ICON_NOTIFY, L"PasswordSafe",
-                               m_LockedIcon, dbox.m_RUEList,
-                               PWS_MSG_ICON_NOTIFY, IDR_POPTRAY);
-  m_TrayIcon->SetTarget(&dbox);
-#endif
-
-  // Set up an Accelerator table
-#if !defined(POCKET_PC)
-  m_ghAccelTable = ::LoadAccelerators(AfxGetResourceHandle(),
-                                      MAKEINTRESOURCE(IDR_ACCS));
+  m_pTrayIcon = new CSystemTray(m_pMainDlg, PWS_MSG_ICON_NOTIFY, L"PasswordSafe",
+                                m_LockedIcon, dbox.m_RUEList,
+                                PWS_MSG_ICON_NOTIFY, IDR_POPTRAY);
+  m_pTrayIcon->SetTarget(&dbox);
 #endif
 
   CLWnd ListenerWnd(dbox);
@@ -1017,6 +1085,10 @@ BOOL ThisMfcApp::InitInstance()
   // Run dialog - note that we don't particularly care what the response was
   (void) dbox.DoModal();
 
+  if (m_pTrayIcon != NULL)
+    m_pTrayIcon->DestroyWindow();
+  delete m_pTrayIcon;
+
   ::DestroyIcon(m_LockedIcon);
   ::DestroyIcon(m_UnLockedIcon);
   ::DestroyIcon(m_ClosedIcon);
@@ -1120,9 +1192,9 @@ int ThisMfcApp::SetClosedTrayIcon(int &iData, bool bSet)
 
 void ThisMfcApp::SetSystemTrayState(STATE s)
 {
-  // need to protect against null m_TrayIcon due to
+  // need to protect against null m_pTrayIcon due to
   // tricky initialization order
-  if (m_TrayIcon != NULL && s != m_TrayLockedState) {
+  if (m_pTrayIcon != NULL && s != m_TrayLockedState) {
     m_TrayLockedState = s;
     HICON hIcon(m_LockedIcon);
     switch (s) {
@@ -1138,7 +1210,7 @@ void ThisMfcApp::SetSystemTrayState(STATE s)
       default:
         break;
     }
-    m_TrayIcon->SetIcon(hIcon);
+    m_pTrayIcon->SetIcon(hIcon);
   }
 }
 
@@ -1275,17 +1347,17 @@ void ThisMfcApp::GetApplicationVersionData()
 
 BOOL CALLBACK ThisMfcApp::searcher(HWND hWnd, LPARAM lParam)
 {
-  DWORD_PTR result;
+  DWORD_PTR dwresult;
   LRESULT ok = ::SendMessageTimeout(hWnd,
                                     m_uiRegMsg,
                                     0, 0, 
                                     SMTO_BLOCK | SMTO_ABORTIFHUNG,
                                     200,
-                                    &result);
+                                    &dwresult);
   if (ok == 0)
     return TRUE; // ignore this and continue
 
-  if (result == m_uiRegMsg) { /* found it */
+  if (dwresult == m_uiRegMsg) { /* found it */
     HWND *target = (HWND *)lParam;
     *target = hWnd;
     return FALSE; // stop search
@@ -1333,33 +1405,31 @@ void ThisMfcApp::GetLanguageFiles()
   // NOTE: Assumption that a Help file can only exist if a resource file also exists
   const CString cs_ExePath(PWSdirs::GetExeDir().c_str());
   CString cs_ResPath = cs_ExePath + L"pwsafe*.dll";
-  //     L"%wsafe%s%s.dll" : L"pwsafe%s_%s.dll";
+  //     L"pwsafeLL.dll" : L"pwsafeLL_CC.dll";
 
   // Find all language DLLs
   m_vlanguagefiles.clear();
 
   LANGHELPFILE st_lng;
+  const StringX sxLL = PWSprefs::GetInstance()->GetPref(PWSprefs::LanguageFile);
 
-  // Add default embedded language
+  // Add default embedded language - English
   const LCID AppLCID = MAKELCID(m_AppLangID, SORT_DEFAULT); // 0x0409 - English US
 
   wchar_t *szLanguage_Native(NULL);
   int inum = ::GetLocaleInfo(AppLCID, LOCALE_SNATIVELANGNAME, szLanguage_Native, 0);
   if (inum > 0) {
-    CString cs_language, cs_Embedded(MAKEINTRESOURCE(IDS_EMBEDDEDRESOURCES));
     szLanguage_Native = new wchar_t[inum + 1];
     ::GetLocaleInfo(AppLCID, LOCALE_SNATIVELANGNAME, szLanguage_Native, inum);
 
-    cs_language.Format(L"%s (%s)", szLanguage_Native, cs_Embedded);
-
     st_lng.lcid = AppLCID;
-    st_lng.xFlags = (m_AppLangID == m_ResLangID) ? 0xC0 : 0x40;
-    st_lng.wsLL = L"";
+    st_lng.xFlags = (wcscmp(sxLL.c_str(), L"EN") == 0) ? 0xC0 : 0x40;
+    st_lng.wsLL = L"EN";
     st_lng.wsCC = L"";
-    st_lng.wsLanguage = (LPCWSTR)cs_language;
+    st_lng.wsLanguage = szLanguage_Native;
 
     m_vlanguagefiles.push_back(st_lng);
-    delete szLanguage_Native;
+    delete[] szLanguage_Native;
   }
 
   CFileFind finder;
@@ -1376,6 +1446,9 @@ void ThisMfcApp::GetLanguageFiles()
     // length should be either 10+2 (pwsafeLL.dll) or 10+5 (pwsafeLL_CC.dll)
     int len = cs_dll.GetLength();
     ASSERT(len == 12 || len == 15);
+    if (len != 12 && len != 15)
+      continue;
+
     cs_LL = cs_dll.Mid(6, 2);
     if (len == 15)
       cs_CC = cs_dll.Mid(9, 2);
@@ -1392,23 +1465,71 @@ void ThisMfcApp::GetLanguageFiles()
     if (FoundResLangID == m_AppLangID)
       continue;
 
+    // Create LCID
     LCID lcid = MAKELCID(FoundResLangID, SORT_DEFAULT);
-    wchar_t *szLanguage_Native(NULL), *szCountry_Language(NULL);
+
+    wchar_t *szLanguage_Native(NULL), *szLanguage_English(NULL);
+    wchar_t *szCountry_Native(NULL), *szCountry_English(NULL);
+
     int inum = ::GetLocaleInfo(lcid, LOCALE_SNATIVELANGNAME, NULL, 0);
     if (inum > 0) {
+      // Get language name in that language
       szLanguage_Native = new wchar_t[inum + 1];
       ::GetLocaleInfo(lcid, LOCALE_SNATIVELANGNAME, szLanguage_Native, inum);
 
-      // For Chinese - we really only want "Chinese (Simplified)" or "Chinese (Traditional)"
-      LCTYPE LCType = LOBYTE(lcid) == 0x04 ? LOCALE_SENGLANGUAGE : LOCALE_SCOUNTRY;
-      int jnum = ::GetLocaleInfo(lcid, LCType, NULL, 0);
-      if (jnum > 0) {
-        szCountry_Language = new wchar_t[jnum + 1];
-        ::GetLocaleInfo(lcid, LCType, szCountry_Language, jnum);
-        if (wcscmp(szLanguage_Native, szCountry_Language) != 0)
-          cs_language.Format(L"%s (%s)", szLanguage_Native, szCountry_Language);
-        else
-          cs_language.Format(L"%s", szLanguage_Native);
+      int jnum = 0, knum = 0;
+      if (!cs_CC.IsEmpty()) {
+        // Get Country name in that language
+        jnum = ::GetLocaleInfo(lcid, LOCALE_SNATIVECTRYNAME, NULL, 0);
+        if (jnum > 0) {
+          szCountry_Native = new wchar_t[jnum + 1];
+          ::GetLocaleInfo(lcid, LOCALE_SNATIVECTRYNAME, szCountry_Native, jnum);
+        }
+        // Get Country name in English
+        knum = ::GetLocaleInfo(lcid, LOCALE_SENGCOUNTRY, NULL, 0);
+        if (knum > 0) {
+          szCountry_English = new wchar_t[knum + 1];
+          ::GetLocaleInfo(lcid, LOCALE_SENGCOUNTRY, szCountry_English, knum);
+        }
+      }
+
+      // Now try to make first character of language name in that language upper case
+      // Should use LCMapStringEx but that is for Vista + and we support XP & later
+      wchar_t *szLanguage_NativeUpper(NULL);
+      int iu = LCMapString(lcid, LCMAP_LINGUISTIC_CASING | LCMAP_UPPERCASE,
+                           szLanguage_Native, inum,
+                           szLanguage_NativeUpper, 0);
+      if (iu > 0) {
+        szLanguage_NativeUpper = new wchar_t[iu + 1];
+        iu = LCMapString(lcid, LCMAP_LINGUISTIC_CASING | LCMAP_UPPERCASE,
+                         szLanguage_Native, inum,
+                         szLanguage_NativeUpper, iu);
+        if (szLanguage_NativeUpper[0] != L'?') {
+          // Assume all OK and language supports Upper case and is read Left->Right
+          // Seems to translate non-Latin characters from Hindi, Punjabi, Hebrew, Korean & Chinese
+          // to the same character - so not issue (and one of them reads Right->Left)
+          // Must be because they do not have a concept of upper & lower case.
+          // Works for Russian Cyrillic alphabet though. Can't promise for future languages!
+          szLanguage_Native[0] = szLanguage_NativeUpper[0];
+        }
+        delete[] szLanguage_NativeUpper;
+      }
+      // Get language name in Englsh
+      int lnum = ::GetLocaleInfo(lcid, LOCALE_SENGLANGUAGE, NULL, 0);
+      if (lnum > 0) {
+        szLanguage_English = new wchar_t[lnum + 1];
+        ::GetLocaleInfo(lcid, LOCALE_SENGLANGUAGE, szLanguage_English, lnum);
+        if (wcscmp(szLanguage_Native, szLanguage_English) != 0) {
+          if (jnum > 0)
+            cs_language.Format(L"%s - %s (%s - %s)", szLanguage_Native, szCountry_Native, szLanguage_English, szCountry_English);
+          else
+            cs_language.Format(L"%s (%s)", szLanguage_Native, szLanguage_English);
+        } else {
+          if (jnum > 0)
+            cs_language.Format(L"%s - %s", szLanguage_Native, szCountry_Native);
+          else
+            cs_language.Format(L"%s", szLanguage_Native);
+        }
       } else
         cs_language.Format(L"%s", szLanguage_Native);
 
@@ -1420,8 +1541,10 @@ void ThisMfcApp::GetLanguageFiles()
       st_lng.wsLanguage = (LPCWSTR)cs_language;
 
       m_vlanguagefiles.push_back(st_lng);
-      delete szLanguage_Native;
-      delete szCountry_Language;
+      delete[] szLanguage_Native;
+      delete[] szLanguage_English;
+      delete[] szCountry_Native;
+      delete[] szCountry_English;
     }
   }
   finder.Close();
