@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -18,7 +18,6 @@
 #include "WZPropertySheet.h"
 
 #include "core/ItemData.h"
-#include "core/core.h"
 
 #include "resource.h"
 #include "resource3.h"
@@ -293,21 +292,22 @@ BOOL CWZAdvanced::OnInitDialog()
       m_pLC_List->SetItemData(iItem, CItemData::XTIME_INT | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::XTIME_INT);
 
-      cs_text.LoadString(IDS_PWPOLICY);
-      iItem = m_pLC_List->InsertItem(++iItem, cs_text);
-      m_pLC_List->SetItemData(iItem, CItemData::POLICY | NORMALFIELD);
-      m_bsAllowedFields.set(CItemData::POLICY);
-
       cs_text.LoadString(IDS_DCALONG);
       iItem = m_pLC_List->InsertItem(++iItem, cs_text);
       m_pLC_List->SetItemData(iItem, CItemData::DCA | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::DCA);
 
+      cs_text.LoadString(IDS_SHIFTDCALONG);
+      iItem = m_pLC_List->InsertItem(++iItem, cs_text);
+      m_pLC_List->SetItemData(iItem, CItemData::SHIFTDCA | NORMALFIELD);
+      m_bsAllowedFields.set(CItemData::SHIFTDCA);
+
       cs_text.LoadString(IDS_PROTECTED);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
       m_pLC_Selected->SetItemData(iItem, CItemData::PROTECTED | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::PROTECTED);
       break;
+
     case WZAdvanced::SYNCH:
     case WZAdvanced::EXPORT_TEXT:
     case WZAdvanced::EXPORT_ENTRYTEXT:
@@ -361,18 +361,31 @@ BOOL CWZAdvanced::OnInitDialog()
       m_bsAllowedFields.set(CItemData::POLICY);
       m_bsDefaultSelectedFields.set(CItemData::POLICY);
 
+      cs_text.LoadString(IDS_POLICYNAME);
+      iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
+      m_pLC_Selected->SetItemData(iItem, CItemData::POLICYNAME | NORMALFIELD);
+      m_bsAllowedFields.set(CItemData::POLICYNAME);
+      m_bsDefaultSelectedFields.set(CItemData::POLICYNAME);
+
       cs_text.LoadString(IDS_DCALONG);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
       m_pLC_Selected->SetItemData(iItem, CItemData::DCA | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::DCA);
       m_bsDefaultSelectedFields.set(CItemData::DCA);
 
+      cs_text.LoadString(IDS_SHIFTDCALONG);
+      iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
+      m_pLC_Selected->SetItemData(iItem, CItemData::SHIFTDCA | NORMALFIELD);
+      m_bsAllowedFields.set(CItemData::SHIFTDCA);
+      m_bsDefaultSelectedFields.set(CItemData::SHIFTDCA);
+
       cs_text.LoadString(IDS_PROTECTED);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
       m_pLC_Selected->SetItemData(iItem, CItemData::PROTECTED | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::PROTECTED);
       m_bsDefaultSelectedFields.set(CItemData::PROTECTED);
       break;
+
     default:
       ASSERT(FALSE);
       break;
@@ -451,6 +464,7 @@ BOOL CWZAdvanced::OnInitDialog()
       m_bsDefaultSelectedFields.set(CItemData::PASSWORD);
       m_bsMandatoryFields.set(CItemData::PASSWORD);
       break;
+
     case WZAdvanced::COMPARE:
       cs_text.LoadString(IDS_ADVANCED_GROUPTEXT); // <-- Special - Mandatory field
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
@@ -478,7 +492,20 @@ BOOL CWZAdvanced::OnInitDialog()
       m_pLC_Selected->SetItemData(iItem, CItemData::PASSWORD | NORMALFIELD);
       m_bsAllowedFields.set(CItemData::PASSWORD);
       m_bsDefaultSelectedFields.set(CItemData::PASSWORD);
+
+      cs_text.LoadString(IDS_PWPOLICY);
+      iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
+      m_pLC_Selected->SetItemData(iItem, CItemData::POLICY | NORMALFIELD);
+      m_bsAllowedFields.set(CItemData::POLICY);
+      m_bsDefaultSelectedFields.set(CItemData::POLICY);
+
+      cs_text.LoadString(IDS_POLICYNAME);
+      iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
+      m_pLC_Selected->SetItemData(iItem, CItemData::POLICYNAME | NORMALFIELD);
+      m_bsAllowedFields.set(CItemData::POLICYNAME);
+      m_bsDefaultSelectedFields.set(CItemData::POLICYNAME);
       break;
+
     case WZAdvanced::SYNCH:
       cs_text.LoadString(IDS_PASSWORD);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
@@ -486,6 +513,7 @@ BOOL CWZAdvanced::OnInitDialog()
       m_bsAllowedFields.set(CItemData::PASSWORD);
       m_bsDefaultSelectedFields.set(CItemData::PASSWORD);
       break;
+
     case WZAdvanced::EXPORT_TEXT:
     case WZAdvanced::EXPORT_ENTRYTEXT:
       cs_text.LoadString(IDS_GROUP);
@@ -512,6 +540,7 @@ BOOL CWZAdvanced::OnInitDialog()
       m_bsAllowedFields.set(CItemData::PASSWORD);
       m_bsDefaultSelectedFields.set(CItemData::PASSWORD);
       break;
+
     default:
       ASSERT(FALSE);
   }
@@ -563,6 +592,7 @@ BOOL CWZAdvanced::OnInitDialog()
   cs_ToolTip.LoadString(IDS_ADVANCED_DESELECTALL);
   m_pToolTipCtrl->AddTool(GetDlgItem(IDC_ADVANCED_DESELECTALL), cs_ToolTip);
 
+
   return TRUE;
 }
 
@@ -794,6 +824,13 @@ LRESULT CWZAdvanced::OnWizardNext()
     }
   }
 
+  // If there are none left in the allowed ListCtrl, then the user must have selected
+  // all items - so set the ones he/she couldn't. This will be the same as if the user
+  // never came to the WSAdvanced dialog
+  m_bsFields.set(CItemData::GROUPTITLE);
+  m_bsFields.set(CItemData::UUID);
+  m_bsFields.set(CItemData::RESERVED);
+
   m_pst_SADV->bsFields = m_bsFields;
   m_pst_SADV->subgroup_name = m_subgroup_name;
   m_pst_SADV->subgroup_bset = m_subgroup_set == BST_CHECKED;
@@ -950,5 +987,42 @@ BOOL CWZAdvanced::PreTranslateMessage(MSG* pMsg)
 int CALLBACK CWZAdvanced::AdvCompareFunc(LPARAM lParam1, LPARAM lParam2,
                                          LPARAM /* closure */)
 {
-  return (int)(lParam1 - lParam2);
+  // Preferred display order
+  // 1. Group, Title, User first - range 0-9
+  // 2. Text fields and fields that are displayed as text, next range 10...49
+  // 3. Non-text fields (not times), next 50...
+  // 4. Times last 100...
+  // 5. Invalid and so shouldn' be here -1
+
+  const int iSortOrder[CItemData::LAST] = {
+      -1 /* NAME       = 0x00 */,
+      -1 /* UUID       = 0x01 */,
+       0 /* GROUP      = 0x02 */,
+       1 /* TITLE      = 0x03 */,
+       2 /* USER       = 0x04 */,
+      10 /* NOTES      = 0x05 */,
+      11 /* PASSWORD   = 0x06 */,
+     100 /* CTIME      = 0x07 */,
+     101 /* PMTIME     = 0x08 */,
+     102 /* ATIME      = 0x09 */,
+     103 /* XTIME      = 0x0a */,
+      -1 /* RESERVED   = 0x0b */,
+     104 /* RMTIME     = 0x0c */,
+      12 /* URL        = 0x0d */,
+      15 /* AUTOTYPE   = 0x0e */,
+      50 /* PWHIST     = 0x0f */,
+      61 /* POLICY     = 0x10 */,
+     505 /* XTIME_INT  = 0x11 */,
+      14 /* RUNCMD     = 0x12 */,
+      81 /* DCA        = 0x13 */,
+      13 /* EMAIL      = 0x14 */,
+      80 /* PROTECTED  = 0x15 */,
+      60 /* SYMBOLS    = 0x16 */,
+      82 /* SHIFTDCA   = 0x17 */,
+      62 /* POLICYNAME = 0x18 */
+  };
+
+  const int i1 = iSortOrder[lParam1 & 0xff];
+  const int i2 = iSortOrder[lParam2 & 0xff];
+  return i1 - i2;
 }
