@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -41,9 +41,14 @@
 #include "../../core/Util.h"
 #include "../../os/KeySend.h"
 #include "../../os/sleep.h"
+#include "../../os/utf8conv.h"
 
 #include <algorithm>
 
+#ifdef __WXMSW__
+#include <wx/msw/msvcrt.h>
+#endif
+
 using pws_os::CUUID;
 
 /*!
@@ -63,7 +68,7 @@ void PasswordSafeFrame::DoEdit(CItemData item)
 {
   int rc = 0;
   if (!item.IsShortcut()) {
-    AddEditPropSheet editDbox(this, m_core, AddEditPropSheet::EDIT, &item);
+    AddEditPropSheet editDbox(this, m_core, AddEditPropSheet::EDIT, &item, this);
     rc = editDbox.ShowModal();
   } else {
     EditShortcut editDbox(this, m_core, &item);
@@ -98,10 +103,10 @@ void PasswordSafeFrame::OnAddClick( wxCommandEvent& /* evt */ )
   wxString selectedGroup = wxEmptyString;
   wxTreeItemId selection;
   if (IsTreeView() && (selection = m_tree->GetSelection()).IsOk() && m_tree->ItemHasChildren(selection)) {
-    selectedGroup = m_tree->GetItemText(selection);
+    selectedGroup = m_tree->GetItemGroup(selection);
   }
 
-  AddEditPropSheet addDbox(this, m_core, AddEditPropSheet::ADD, NULL, selectedGroup);
+  AddEditPropSheet addDbox(this, m_core, AddEditPropSheet::ADD, NULL, this, selectedGroup);
   if (addDbox.ShowModal() == wxID_OK) {
     const CItemData &item = addDbox.GetItem();
     m_core.Execute(AddEntryCommand::Create(&m_core, item));
@@ -447,6 +452,12 @@ void PasswordSafeFrame::DoCopyEmail(CItemData &item)
  */
 void PasswordSafeFrame::DoAutotype(CItemData &ci)
 {
+  std::vector<size_t> vactionverboffsets;
+  const StringX sxautotype = PWSAuxParse::GetAutoTypeString(ci, m_core,
+                                                            vactionverboffsets);
+
+  UpdateAccessTime(ci);
+
   // Called from OnAutoType and OnTrayAutoType
 
   // Rules are ("Minimize on Autotype" takes precedence):
@@ -472,18 +483,27 @@ void PasswordSafeFrame::DoAutotype(CItemData &ci)
     m_guiInfo->Save(this);
     Hide();
   }
- 
-  std::vector<size_t> vactionverboffsets;
-  const StringX sxautotype = PWSAuxParse::GetAutoTypeString(ci, m_core,
-                                                            vactionverboffsets);
-  DoAutotype(sxautotype, vactionverboffsets);
-  UpdateAccessTime(ci);
 
-  // If we minimized it, exit. If we only hid it, now show it
-  if (bMinOnAuto)
-    return;
+  bool autotype_err = false;
+  wxString autotype_err_msg;
+  try {
+    DoAutotype(sxautotype, vactionverboffsets);
+  }
+  catch(const std::exception& e) {
+    autotype_err = true;
+#ifndef _WIN32
+    // pws_os::towc is not defined for Windows
+    autotype_err_msg = towxstring(pws_os::towc(e.what()));
+#else
+    UNREFERENCED_PARAMETER(e);
+#endif
+  }
 
-  if (PWSprefs::GetInstance()->GetPref(PWSprefs::AlwaysOnTop)) {
+  // Restore the UI if
+  //   1. there was an error autotyping
+  //   2. We're supposed to be always-on-top
+  //   3. We hid the UI instead of minimizing while autotyping
+  if (autotype_err || PWSprefs::GetInstance()->GetPref(PWSprefs::AlwaysOnTop) || !bMinOnAuto) {
     /* TODO - figure out how to keep a wxWidgets window always on top */
     if (IsIconized())
       Iconize(false);
@@ -492,6 +512,8 @@ void PasswordSafeFrame::DoAutotype(CItemData &ci)
       m_guiInfo->Restore(this);
     }
   }
+  if (autotype_err)
+    wxMessageBox(_("There was an error autotyping.  ") + autotype_err_msg, _T("Autotype error"), wxOK|wxICON_ERROR, this);
 }
 
 /*
